@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Code generation for locals and globals handling.
 
 This also includes writing back to locals for exec statements.
@@ -49,7 +35,6 @@ def generateBuiltinLocalsRefCode(to_name, expression, emit, context):
     with withObjectCodeTemporaryAssignment(
         to_name, "locals_ref_value", expression, emit, context
     ) as value_name:
-
         emit("%s = %s;" % (value_name, locals_declaration))
 
 
@@ -69,7 +54,6 @@ def generateBuiltinLocalsCode(to_name, expression, emit, context):
     with withObjectCodeTemporaryAssignment(
         to_name, "locals_ref_value", expression, emit, context
     ) as value_name:
-
         if updated:
             locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
             is_dict = locals_scope.hasShapeDictionaryExact()
@@ -187,6 +171,7 @@ def generateBuiltinDir1Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="PyObject_Dir",
+        tstate=False,
         arg_desc=(("dir_arg", expression.subnode_value),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -200,6 +185,7 @@ def generateBuiltinVarsCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="LOOKUP_VARS",
+        tstate=True,
         arg_desc=(("vars_arg", expression.subnode_source),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -207,3 +193,19 @@ def generateBuiltinVarsCode(to_name, expression, emit, context):
         emit=emit,
         context=context,
     )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
