@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Code generation for lists.
 
 Right now only the creation is done here. But more should be added later on.
@@ -162,10 +148,10 @@ def generateListOperationExtendCode(to_name, expression, emit, context):
     res_name = context.getBoolResName()
 
     emit(
-        "%s = %s(%s, %s);"
+        "%s = %s(tstate, %s, %s);"
         % (
             res_name,
-            "LIST_EXTEND_FOR_UNPACK" if is_unpack else "LIST_EXTEND",
+            "LIST_EXTEND_FOR_UNPACK" if is_unpack else "LIST_EXTEND_FROM_ITERABLE",
             list_arg_name,
             value_arg_name,
         )
@@ -234,7 +220,10 @@ def generateListOperationIndex2Code(to_name, expression, emit, context):
     with withObjectCodeTemporaryAssignment(
         to_name, "list_index_result", expression, emit, context
     ) as result_name:
-        emit("%s = LIST_INDEX2(%s, %s);" % (result_name, list_arg_name, value_arg_name))
+        emit(
+            "%s = LIST_INDEX2(tstate, %s, %s);"
+            % (result_name, list_arg_name, value_arg_name)
+        )
 
         getErrorExitCode(
             check_name=result_name,
@@ -256,7 +245,7 @@ def generateListOperationIndex3Code(to_name, expression, emit, context):
         to_name, "list_index_result", expression, emit, context
     ) as result_name:
         emit(
-            "%s = LIST_INDEX3(%s, %s, %s);"
+            "%s = LIST_INDEX3(tstate, %s, %s, %s);"
             % (result_name, list_arg_name, value_arg_name, start_arg_name)
         )
 
@@ -283,7 +272,7 @@ def generateListOperationIndex4Code(to_name, expression, emit, context):
         to_name, "list_index_result", expression, emit, context
     ) as result_name:
         emit(
-            "%s = LIST_INDEX4(%s, %s, %s, %s);"
+            "%s = LIST_INDEX4(tstate, %s, %s, %s, %s);"
             % (
                 result_name,
                 list_arg_name,
@@ -347,7 +336,7 @@ def generateListOperationInsertCode(to_name, expression, emit, context):
         res_name = context.getBoolResName()
 
         emit(
-            "%s = LIST_INSERT(%s, %s, %s);"
+            "%s = LIST_INSERT(tstate, %s, %s, %s);"
             % (
                 res_name,
                 list_arg_name,
@@ -396,6 +385,7 @@ def generateListOperationPop1Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="LIST_POP1",
+        tstate=True,
         arg_desc=makeArgDescFromExpression(expression),
         may_raise=expression.mayRaiseExceptionOperation(),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -409,6 +399,7 @@ def generateListOperationPop2Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="LIST_POP2",
+        tstate=True,
         arg_desc=makeArgDescFromExpression(expression),
         may_raise=expression.mayRaiseExceptionOperation(),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -423,32 +414,26 @@ def generateListOperationRemoveCode(to_name, expression, emit, context):
         expression=expression, emit=emit, context=context
     )
 
-    with withObjectCodeTemporaryAssignment(
-        to_name, "list_remove_result", expression, emit, context
-    ) as result_name:
+    res_name = context.getBoolResName()
 
-        # TODO: Have a dedicated list helper instead, this could be more efficient,
-        # this call is also very bad.
-        emit("assert(PyList_CheckExact(%s));" % list_arg_name)
-        emit(
-            '%s = PyObject_CallMethod(%s, (char *)"remove", "O", %s);'
-            % (result_name, list_arg_name, value_arg_name)
-        )
+    emit("%s = LIST_REMOVE(%s, %s);" % (res_name, list_arg_name, value_arg_name))
 
-        getErrorExitCode(
-            check_name=result_name,
-            release_names=(list_arg_name, value_arg_name),
-            emit=emit,
-            context=context,
-        )
+    getErrorExitBoolCode(
+        condition="%s == false" % res_name,
+        release_names=(list_arg_name, value_arg_name),
+        needs_check=expression.mayRaiseExceptionOperation(),
+        emit=emit,
+        context=context,
+    )
 
-        context.addCleanupTempName(result_name)
+    assignConstantNoneResult(to_name, emit, context)
 
 
 def generateListOperationSort1Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="LIST_SORT1",
+        tstate=True,
         arg_desc=makeArgDescFromExpression(expression),
         may_raise=expression.mayRaiseExceptionOperation(),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -462,6 +447,7 @@ def generateListOperationSort2Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="LIST_SORT2",
+        tstate=True,
         arg_desc=makeArgDescFromExpression(expression),
         may_raise=expression.mayRaiseExceptionOperation(),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -475,6 +461,7 @@ def generateListOperationSort3Code(to_name, expression, emit, context):
     generateCAPIObjectCode0(
         to_name=to_name,
         capi="LIST_SORT3",
+        tstate=True,
         arg_desc=makeArgDescFromExpression(expression),
         may_raise=expression.mayRaiseExceptionOperation(),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -489,6 +476,7 @@ def generateBuiltinListCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="MAKE_LIST",
+        tstate=True,
         arg_desc=(("list_arg", expression.subnode_value),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -496,3 +484,19 @@ def generateBuiltinListCode(to_name, expression, emit, context):
         emit=emit,
         context=context,
     )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
