@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Namify constants.
 
 This determines the identifier names of constants in the generated code. We
@@ -23,14 +9,12 @@ it is really necessary.
 
 """
 
-
-import hashlib
 import math
 import re
 import sys
 from types import BuiltinFunctionType
 
-from nuitka.__past__ import GenericAlias, UnionType, long, unicode, xrange
+from nuitka.__past__ import GenericAlias, UnionType, long, md5, unicode, xrange
 from nuitka.Builtins import builtin_anon_values, builtin_named_values_list
 from nuitka.Tracing import general
 
@@ -198,6 +182,8 @@ def _namifyString(string):
 
     if string in ("", b""):
         return "empty"
+    elif string in ("\0", b"\0"):
+        return "null"
     elif string == " ":
         return "space"
     elif string == ".":
@@ -208,6 +194,8 @@ def _namifyString(string):
         return "slash"
     elif string == "\\":
         return "backslash"
+    elif string == "_":
+        return "underscore"
     elif (
         type(string) is str
         and _re_str_needs_no_digest.match(string)
@@ -242,10 +230,26 @@ def _isAscii(string):
 def _digest(value):
     if str is bytes:
         # Python2 is simple
-        return hashlib.md5(value).hexdigest()
+        return md5(value).hexdigest()
     else:
         # Python3 needs to encode the string if it is one.
         if type(value) is bytes:
-            return hashlib.md5(value).hexdigest()
+            return md5(value).hexdigest()
         else:
-            return hashlib.md5(value.encode("utf8")).hexdigest()
+            return md5(value.encode("utf8")).hexdigest()
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
