@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """Code generation for package resources access."""
 
 from nuitka.Options import shallMakeModule
@@ -180,7 +166,7 @@ def generateImportlibMetadataDistributionValueCode(to_name, expression, emit, co
         to_name, "distribution_value", expression, emit, context
     ) as value_name:
         emit(
-            """%(to_name)s = Nuitka_Distribution_New(%(name)s);"""
+            """%(to_name)s = Nuitka_Distribution_New(tstate, %(name)s);"""
             % {
                 "to_name": value_name,
                 "name": context.getConstantCode(original_name),
@@ -534,11 +520,13 @@ def generatePkgResourcesIterEntryPointsCallCode(to_name, expression, emit, conte
             called_name=iter_entry_points_function_name,
             expression=expression,
             arg_names=(
-                group_arg_name,
-                name_arg_name,
-            )
-            if name_arg_name is not None
-            else (group_arg_name,),
+                (
+                    group_arg_name,
+                    name_arg_name,
+                )
+                if name_arg_name is not None
+                else (group_arg_name,)
+            ),
             emit=emit,
             context=context,
         )
@@ -748,7 +736,7 @@ def generateImportlibResourcesFilesCallCode(to_name, expression, emit, context):
 
         getImportModuleNameHardCode(
             to_name=files_function,
-            module_name="importlib.resources",
+            module_name=expression.importlib_resources_name,
             import_name="files",
             needs_check=False,
             emit=emit,
@@ -824,6 +812,7 @@ def generateOsPathExistsCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_PATH_FILE_EXISTS",
+        tstate=True,
         arg_desc=(("exists_arg", expression.subnode_path),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -837,6 +826,7 @@ def generateOsPathIsfileCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_PATH_FILE_ISFILE",
+        tstate=True,
         arg_desc=(("isfile_arg", expression.subnode_path),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -850,6 +840,7 @@ def generateOsPathIsdirCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_PATH_FILE_ISDIR",
+        tstate=True,
         arg_desc=(("isdir_arg", expression.subnode_path),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -863,6 +854,21 @@ def generateOsPathBasenameCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_PATH_BASENAME",
+        tstate=True,
+        arg_desc=(("path_arg", expression.subnode_p),),
+        may_raise=expression.mayRaiseException(BaseException),
+        conversion_check=decideConversionCheckNeeded(to_name, expression),
+        source_ref=expression.getCompatibleSourceReference(),
+        emit=emit,
+        context=context,
+    )
+
+
+def generateOsPathDirnameCallCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name=to_name,
+        capi="OS_PATH_DIRNAME",
+        tstate=True,
         arg_desc=(("path_arg", expression.subnode_p),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -876,6 +882,7 @@ def generateOsPathAbspathCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_PATH_ABSPATH",
+        tstate=True,
         arg_desc=(("path_arg", expression.subnode_path),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -889,6 +896,7 @@ def generateOsPathIsabsCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_PATH_ISABS",
+        tstate=True,
         arg_desc=(("path_arg", expression.subnode_s),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -902,6 +910,7 @@ def generateOsListdirCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
         capi="OS_LISTDIR",
+        tstate=True,
         arg_desc=(("path_arg", expression.subnode_path),),
         may_raise=expression.mayRaiseException(BaseException),
         conversion_check=decideConversionCheckNeeded(to_name, expression),
@@ -910,3 +919,19 @@ def generateOsListdirCallCode(to_name, expression, emit, context):
         emit=emit,
         context=context,
     )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
