@@ -25,7 +25,7 @@ PyObject *Nuitka_dunder_compiled_value = NULL;
 #ifdef _NUITKA_STANDALONE
 extern PyObject *getStandaloneSysExecutablePath(PyObject *basename);
 
-static PyObject *STRIP_DIRNAME(PyObject *path) {
+NUITKA_MAY_BE_UNUSED static PyObject *STRIP_DIRNAME(PyObject *path) {
 #if PYTHON_VERSION < 0x300
     char const *path_cstr = PyString_AS_STRING(path);
 
@@ -127,6 +127,7 @@ static void _createGlobalConstants(PyThreadState *tstate) {
         {(char *)"no_docstrings", (char *)"boolean indicating --python-flag=no_docstrings usage"},
         {(char *)"no_annotations", (char *)"boolean indicating --python-flag=no_annotations usage"},
         {(char *)"module", (char *)"boolean indicating --module usage"},
+        {(char *)"main", (char *)"name of main module at runtime"},
         {0}
     };
 
@@ -149,6 +150,7 @@ static void _createGlobalConstants(PyThreadState *tstate) {
     PyStructSequence_SET_ITEM(Nuitka_dunder_compiled_value, 3, Nuitka_String_FromString("%(nuitka_version_level)s"));
 
     PyObject *binary_directory = getContainingDirectoryObject(false);
+    binary_directory = OS_PATH_ABSPATH(tstate, binary_directory);
 #ifdef _NUITKA_STANDALONE
 #ifndef _NUITKA_ONEFILE_MODE
     binary_directory = STRIP_DIRNAME(binary_directory);
@@ -210,6 +212,19 @@ static void _createGlobalConstants(PyThreadState *tstate) {
 #endif
     PyStructSequence_SET_ITEM(Nuitka_dunder_compiled_value, 11, is_module_mode);
 
+#ifdef _NUITKA_MODULE
+    PyObject *main_name;
+
+    if (_Py_PackageContext != NULL) {
+        main_name = Nuitka_String_FromString(_Py_PackageContext);
+    } else {
+        main_name = Nuitka_String_FromString(%(module_name_cstr)s);
+    }
+#else
+    PyObject *main_name = Nuitka_String_FromString("__main__");
+#endif
+    PyStructSequence_SET_ITEM(Nuitka_dunder_compiled_value, 12, main_name);
+
     // Prevent users from creating the Nuitka version type object.
     Nuitka_VersionInfoType.tp_init = NULL;
     Nuitka_VersionInfoType.tp_new = NULL;
