@@ -1,23 +1,10 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """DLL dependency scan methods for macOS. """
 
 import os
+import re
 import sys
 
 from nuitka.containers.OrderedDicts import OrderedDict
@@ -27,7 +14,11 @@ from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonFlavors import isAnacondaPython, isNuitkaPython
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import inclusion_logger
-from nuitka.utils.FileOperations import areSamePaths, isFilenameBelowPath
+from nuitka.utils.FileOperations import (
+    areSamePaths,
+    getReportPath,
+    isFilenameBelowPath,
+)
 from nuitka.utils.Importing import getSharedLibrarySuffixes
 from nuitka.utils.Json import loadJsonFromFilename
 from nuitka.utils.SharedLibraries import (
@@ -35,6 +26,7 @@ from nuitka.utils.SharedLibraries import (
     getOtoolDependencyOutput,
     getOtoolListing,
 )
+from nuitka.utils.Utils import getArchitecture
 
 from .DllDependenciesCommon import getLdLibraryPath
 
@@ -59,7 +51,7 @@ def _detectPythonRpaths():
         if os.path.isdir(candidate):
             result.append(candidate)
 
-    return tuple(result)
+    return tuple(set(result))
 
 
 def detectBinaryPathDLLsMacOS(
@@ -102,8 +94,8 @@ def detectBinaryPathDLLsMacOS(
                 original_dir=os.path.dirname(sub_dll_filename),
                 binary_filename=sub_dll_filename,
                 package_name=package_name,
-                recursive=True,
                 keep_unresolved=True,
+                recursive=True,
             )
 
             merged_result.update(sub_result)
@@ -142,11 +134,29 @@ def _parseOtoolListingOutput(output):
     return paths
 
 
+def _getNonVersionedDllFilenames(filename):
+    yield filename
+
+    if getArchitecture() == "arm64" and filename.endswith(".dylib"):
+        yield filename[:-6] + "_arm64.dylib"
+
+    match = re.match(r"^(.*?)(\.\d+)+\.dylib$", filename)
+
+    if match:
+        yield match.group(1) + ".dylib"
+
+        # TODO: The versioned filename, might do the same, and so could the
+        # "x86_64" specific DLL be a thing too, but we should have actual
+        # examples to be sure they are covered with tests.
+        if getArchitecture() == "arm64":
+            yield match.group(1) + "_arm64.dylib"
+
+
 def _resolveBinaryPathDLLsMacOS(
     original_dir, binary_filename, paths, package_specific_dirs, package_name
 ):
     # Quite a few variations to consider
-    # pylint: disable=too-many-branches,too-many-locals
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     had_self = False
 
@@ -158,11 +168,17 @@ def _resolveBinaryPathDLLsMacOS(
     for path in paths:
         if path.startswith("@rpath/"):
             # Resolve rpath to just the ones given, first match.
+            library_name = path[7:]
+
             for rpath in rpaths:
-                if os.path.exists(os.path.join(rpath, path[7:])):
+                if os.path.exists(os.path.join(rpath, library_name)):
                     resolved_path = os.path.normpath(os.path.join(rpath, path[7:]))
                     break
             else:
+                # These have become virtual in later macOS.
+                if library_name in ("libc++.1.dylib", "libz.1.dylib"):
+                    continue
+
                 # This is only a guess, might be missing package specific directories.
                 resolved_path = os.path.normpath(os.path.join(original_dir, path[7:]))
         elif path.startswith("@loader_path/"):
@@ -186,23 +202,71 @@ def _resolveBinaryPathDLLsMacOS(
         else:
             resolved_path = path
 
+        # Some extension modules seem to reference themselves a wrong name,
+        # duplicating their module name into the filename, but that does
+        # not exist.
+        if not os.path.exists(resolved_path) and package_name is not None:
+            parts = os.path.basename(resolved_path).split(".")
+
+            if parts[0] == package_name.asString():
+                resolved_path = os.path.join(
+                    os.path.dirname(resolved_path), ".".join(parts[1:])
+                )
+
         # Some extension modules seem to reference themselves by a different
-        # extension module name, so use that if it exists.
-        if not os.path.exists(resolved_path) and python_version >= 0x300:
+        # extension module name, so use that if it exists, and some versioned
+        # DLL dependencies do not matter.
+        if python_version >= 0x300:
             so_suffixes = getSharedLibrarySuffixes()[:-1]
 
             specific_suffix = so_suffixes[0]
             abi_suffix = so_suffixes[1]
 
-            if resolved_path.endswith(specific_suffix):
-                candidate = resolved_path[: -len(specific_suffix)] + abi_suffix
-            elif resolved_path.endswith(abi_suffix):
-                candidate = resolved_path[: -len(specific_suffix)] + abi_suffix
-            else:
-                candidate = None
+            for resolved_path_candidate in _getNonVersionedDllFilenames(resolved_path):
+                if os.path.exists(resolved_path_candidate):
+                    resolved_path = resolved_path_candidate
+                    break
 
-            if candidate is not None and os.path.exists(candidate):
-                resolved_path = candidate
+                if resolved_path_candidate.endswith(specific_suffix):
+                    candidate = (
+                        resolved_path_candidate[: -len(specific_suffix)] + abi_suffix
+                    )
+                elif resolved_path_candidate.endswith(abi_suffix):
+                    candidate = (
+                        resolved_path_candidate[: -len(specific_suffix)] + abi_suffix
+                    )
+                else:
+                    candidate = None
+
+                if candidate is not None and os.path.exists(candidate):
+                    resolved_path = candidate
+                    break
+
+        # Sometimes self-dependencies are on a numbered version, but deployed is
+        # one version without it. The macOS just ignores that, and so we do.
+        if not os.path.exists(resolved_path):
+            if os.path.basename(resolved_path) == os.path.basename(binary_filename):
+                resolved_path = binary_filename
+            else:
+                match = re.match(r"^(.*?)(\.\d+)+\.dylib$", resolved_path)
+
+                if match:
+                    candidate = match.group(1) + ".dylib"
+
+                    if os.path.exists(candidate):
+                        resolved_path = candidate
+                    else:
+                        candidate = os.path.join(
+                            original_dir, os.path.basename(candidate)
+                        )
+
+                        if os.path.exists(candidate):
+                            resolved_path = candidate
+                        elif os.path.basename(candidate) == os.path.basename(
+                            binary_filename
+                        ):
+                            # Versioned dependency on itself in non-existent path.
+                            resolved_path = binary_filename
 
         if not os.path.exists(resolved_path):
             acceptable, plugin_name = Plugins.isAcceptableMissingDLL(
@@ -213,6 +277,17 @@ def _resolveBinaryPathDLLsMacOS(
             # TODO: Missing DLLs that are accepted, are not really forbidden, we
             # should instead acknowledge them as missing, and treat that properly
             # in using code.
+            if acceptable is True:
+                raise NuitkaForbiddenDLLEncounter(binary_filename, plugin_name)
+
+            # We check both the user and the used DLL if they are listed. This
+            # might be a form of bug hiding, that the later is not sufficient,
+            # that we should address later.
+            acceptable, plugin_name = Plugins.isAcceptableMissingDLL(
+                package_name=package_name,
+                filename=resolved_path,
+            )
+
             if acceptable is True:
                 raise NuitkaForbiddenDLLEncounter(binary_filename, plugin_name)
 
@@ -290,7 +365,7 @@ def fixupBinaryDLLPathsMacOS(
                 inclusion_logger.sysexit(
                     """\
     Error, problem with dependency scan of '%s' with '%s' please report the bug."""
-                    % (original_location, rpath_filename)
+                    % (getReportPath(original_location), rpath_filename)
                 )
 
             mapping.append((rpath_filename, "@executable_path/" + dist_path))
@@ -302,3 +377,19 @@ def fixupBinaryDLLPathsMacOS(
                 id_path=os.path.basename(binary_filename) if had_self else None,
                 rpath=None,
             )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
