@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Included data files for standalone mode.
 
 This keeps track of data files for standalone mode. Do not should them for
@@ -28,10 +14,13 @@ import os
 from nuitka.containers.OrderedDicts import OrderedDict
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.Options import (
+    getOutputPath,
     getShallIncludeDataDirs,
     getShallIncludeDataFiles,
+    getShallIncludeExternallyDataFilePatterns,
     getShallIncludePackageData,
     getShallNotIncludeDataFilePatterns,
+    isAcceleratedMode,
     isOnefileMode,
     isStandaloneMode,
     shallMakeModule,
@@ -39,6 +28,7 @@ from nuitka.Options import (
 from nuitka.OutputDirectories import getStandaloneDirectoryPath
 from nuitka.Tracing import options_logger
 from nuitka.utils.FileOperations import (
+    areSamePaths,
     containsPathElements,
     copyFileWithPermissions,
     getFileContents,
@@ -90,6 +80,15 @@ class IncludedDataFile(object):
 
     def __init__(self, kind, source_path, dest_path, reason, data, tags, tracer):
         tags_set = getDataFileTags(dest_path)
+
+        current = dest_path
+        while True:
+            current = os.path.dirname(current)
+            if not current:
+                break
+
+            tags_set.update(getDataFileTags(current))
+
         tags_set.update(decodeDataFileTags(tags))
 
         # Copy, unless specified otherwise.
@@ -105,7 +104,7 @@ class IncludedDataFile(object):
         self.tracer = tracer
 
     def __repr__(self):
-        return "<%s %s source %s dest %s reason '%s' tags '%s'>" % (
+        return "<%s %s source '%s' dest '%s' reason '%s' tags '%s'>" % (
             self.__class__.__name__,
             self.kind,
             self.source_path,
@@ -148,18 +147,6 @@ def makeIncludedDataFile(source_path, dest_path, reason, tracer, tags):
     if "framework_resource" in tags and not isMacOS():
         tracer.sysexit("Using resource files on non-MacOS")
 
-    inside = True
-    if not isRelativePath(dest_path):
-        if "framework_resource" in tags and not isOnefileMode():
-            inside = isRelativePath(os.path.join("Resources", dest_path))
-        else:
-            inside = False
-
-    if not inside:
-        tracer.sysexit(
-            "Error, cannot use dest path '%s' outside of distribution." % dest_path
-        )
-
     # Refuse directories, these must be kept distinct.
     if os.path.isdir(source_path):
         tracer.sysexit(
@@ -167,6 +154,27 @@ def makeIncludedDataFile(source_path, dest_path, reason, tracer, tags):
             % source_path
         )
 
+    # In accelerated mode, data files can be everywhere, but they cannot
+    # change place.
+    if isAcceleratedMode():
+        if "package_data" not in tags and not areSamePaths(source_path, dest_path):
+            tracer.sysexit(
+                "Error, cannot change paths for data files in accelerated mode '%s'."
+                % dest_path
+            )
+    else:
+        inside = True
+        if not isRelativePath(dest_path):
+            if "framework_resource" in tags and not isOnefileMode():
+                inside = isRelativePath(os.path.join("Resources", dest_path))
+            else:
+                inside = False
+
+        if not inside:
+            tracer.sysexit(
+                "Error, cannot use dest path '%s' outside of distribution." % dest_path
+            )
+
     return IncludedDataFile(
         kind="data_file",
         source_path=source_path,
@@ -187,8 +195,11 @@ default_ignored_suffixes = (
     ".pyi",
     ".so",
     ".pyd",
+    ".pyx",
     ".dll",
     ".dylib",
+    ".exe",
+    ".bin",
 )
 
 default_ignored_suffixes += getSharedLibrarySuffixes()
@@ -196,6 +207,8 @@ default_ignored_suffixes += getSharedLibrarySuffixes()
 default_ignored_dirs = (
     "__pycache__",
     "site-packages",
+    "dist-packages",
+    "vendor-packages",
 )
 
 default_ignored_filenames = ("py.typed",)
@@ -272,11 +285,11 @@ _included_data_files = []
 def addIncludedDataFile(included_datafile):
     included_datafile.tags.update(getDataFileTags(included_datafile.dest_path))
 
-    for noinclude_datafile_pattern in getShallNotIncludeDataFilePatterns():
+    for external_datafile_pattern in getShallNotIncludeDataFilePatterns():
         if fnmatch.fnmatch(
-            included_datafile.dest_path, noinclude_datafile_pattern
+            included_datafile.dest_path, external_datafile_pattern
         ) or isFilenameBelowPath(
-            path=noinclude_datafile_pattern, filename=included_datafile.dest_path
+            path=external_datafile_pattern, filename=included_datafile.dest_path
         ):
             included_datafile.tags.add("inhibit")
             included_datafile.tags.remove("copy")
@@ -293,6 +306,15 @@ def addIncludedDataFile(included_datafile):
     #     if candidate.dest_path == included_datafile.dest_path:
     #         assert False, included_datafile
 
+    if included_datafile.needsCopy():
+        for external_datafile_pattern in getShallIncludeExternallyDataFilePatterns():
+            if fnmatch.fnmatch(
+                included_datafile.dest_path, external_datafile_pattern
+            ) or isFilenameBelowPath(
+                path=external_datafile_pattern, filename=included_datafile.dest_path
+            ):
+                included_datafile.tags.add("external")
+
     _included_data_files.append(included_datafile)
 
 
@@ -495,7 +517,25 @@ def _reportDataFiles():
     _data_file_traces.clear()
 
 
-def _handleDataFile(included_datafile):
+def _checkPathConflict(dest_path, standalone_entry_points):
+    assert os.path.normpath(dest_path) == dest_path
+
+    while dest_path:
+        for standalone_entry_point in standalone_entry_points:
+            if dest_path == standalone_entry_point.dest_path:
+                options_logger.sysexit(
+                    """\
+Error, data file to be placed in distribution as '%s' conflicts with %s '%s'."""
+                    % (
+                        dest_path,
+                        standalone_entry_point.kind,
+                        standalone_entry_point.dest_path,
+                    )
+                )
+        dest_path = os.path.dirname(dest_path)
+
+
+def _handleDataFile(included_datafile, standalone_entry_points):
     """Handle a data file."""
     tracer = included_datafile.tracer
 
@@ -507,24 +547,28 @@ def _handleDataFile(included_datafile):
             "Error, package data files are only included in standalone or onefile mode."
         )
 
-    dist_dir = getStandaloneDirectoryPath()
-
     key = tracer, included_datafile.reason
     if key not in _data_file_traces:
         _data_file_traces[key] = []
 
     _data_file_traces[key].append((included_datafile.kind, included_datafile.dest_path))
 
-    if included_datafile.kind == "data_blob":
+    dist_dir = getStandaloneDirectoryPath()
+
+    if "external" in included_datafile.tags:
+        dest_path = getOutputPath(included_datafile.dest_path)
+    else:
+        _checkPathConflict(included_datafile.dest_path, standalone_entry_points)
         dest_path = os.path.join(dist_dir, included_datafile.dest_path)
+
+    if included_datafile.kind == "data_blob":
         makePath(os.path.dirname(dest_path))
 
         with openTextFile(filename=dest_path, mode="wb") as output_file:
             output_file.write(included_datafile.data)
     elif included_datafile.kind == "data_file":
-        dest_path = os.path.join(dist_dir, included_datafile.dest_path)
-
         makePath(os.path.dirname(dest_path))
+
         copyFileWithPermissions(
             source_path=included_datafile.source_path,
             dest_path=dest_path,
@@ -534,7 +578,7 @@ def _handleDataFile(included_datafile):
         assert False, included_datafile
 
 
-def copyDataFiles():
+def copyDataFiles(standalone_entry_points):
     """Copy the data files needed for standalone distribution.
 
     Notes:
@@ -562,7 +606,24 @@ plugins '--embed-*' options. Not done for '%s'."""
                 )
 
             _handleDataFile(
-                included_datafile,
+                included_datafile=included_datafile,
+                standalone_entry_points=standalone_entry_points,
             )
 
     _reportDataFiles()
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
