@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Nodes for functions and their creations.
 
 Lambdas are functions too. The functions are at the core of the language and
@@ -30,6 +16,7 @@ classes.
 """
 
 import inspect
+import re
 
 from nuitka import Options, Variables
 from nuitka.Constants import isMutable
@@ -43,8 +30,9 @@ from nuitka.specs.ParameterSpecs import (
     TooManyArguments,
     matchCall,
 )
-from nuitka.Tracing import optimization_logger
+from nuitka.Tracing import optimization_logger, printError
 from nuitka.tree.Extractions import updateVariableUsage
+from nuitka.tree.SourceHandling import readSourceLines
 from nuitka.tree.TreeHelpers import makeDictCreationOrConstant2
 
 from .ChildrenHavingMixins import (
@@ -396,7 +384,6 @@ class ExpressionFunctionBodyBase(
         changed = False
 
         for closure_variable in self.getClosureVariables():
-
             # Need to take closure of those either way
             if (
                 closure_variable.isParameterVariable()
@@ -569,7 +556,7 @@ class ExpressionFunctionBody(
     ExpressionFunctionEntryPointBase,
 ):
     # TODO: There should be more special ones than this general type in order to
-    # not cover exec ones in the same object. pylint: disable=too-many-instance-attributes
+    # not cover exec ones in the same object.
 
     kind = "EXPRESSION_FUNCTION_BODY"
 
@@ -821,9 +808,11 @@ class ExpressionFunctionPureBody(ExpressionFunctionBody):
                     "{source_ref} : {tags} : {message}".format(
                         source_ref=source_ref.getAsString(),
                         tags=tag,
-                        message=change_desc()
-                        if inspect.isfunction(change_desc)
-                        else change_desc,
+                        message=(
+                            change_desc()
+                            if inspect.isfunction(change_desc)
+                            else change_desc
+                        ),
                     )
                 )
 
@@ -866,8 +855,6 @@ def makeExpressionFunctionCreation(
     if kw_defaults is not None and kw_defaults.isExpressionConstantDictEmptyRef():
         kw_defaults = None
 
-    assert kw_defaults is None or kw_defaults.isExpression()
-    assert annotations is None or annotations.isExpression()
     assert function_ref.isExpressionFunctionRef()
 
     return ExpressionFunctionCreation(
@@ -933,7 +920,6 @@ class ExpressionFunctionCreationMixin(SideEffectsFromChildrenMixin):
         return False
 
     def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
-
         trace_collection.onExceptionRaiseExit(BaseException)
 
         # TODO: Until we have something to re-order the keyword arguments, we
@@ -1118,7 +1104,7 @@ if python_version < 0x340:
 class ExpressionFunctionRef(ExpressionNoSideEffectsMixin, ExpressionBase):
     kind = "EXPRESSION_FUNCTION_REF"
 
-    __slots__ = "function_body", "code_name"
+    __slots__ = "function_body", "code_name", "function_source"
 
     def __init__(self, source_ref, function_body=None, code_name=None):
         assert function_body is not None or code_name is not None
@@ -1128,10 +1114,12 @@ class ExpressionFunctionRef(ExpressionNoSideEffectsMixin, ExpressionBase):
 
         self.function_body = function_body
         self.code_name = code_name
+        self.function_source = None
 
     def finalize(self):
         del self.parent
         del self.function_body
+        del self.function_source
 
     def getName(self):
         return self.function_body.getName()
@@ -1160,6 +1148,45 @@ class ExpressionFunctionRef(ExpressionNoSideEffectsMixin, ExpressionBase):
         # TODO: Function after collection may now know something.
         return self, None, None
 
+    def getFunctionSourceCode(self):
+        if self.function_source is None:
+            try:
+                lines = readSourceLines(self.getFunctionBody().source_ref)
+
+                # This needs to match "inspect.findsource".
+                pat = re.compile(
+                    r"^(\s*def\s)|(\s*async\s+def\s)|(.*(?<!\w)lambda(:|\s))|^(\s*@)"
+                )
+
+                line_number = self.source_ref.line - 1
+
+                while line_number > 0:
+                    line = lines[line_number]
+
+                    if pat.match(line):
+                        break
+
+                    line_number = line_number - 1
+
+                self.function_source = (
+                    "".join(inspect.getblock(lines[line_number:])),
+                    line_number,
+                )
+            except Exception:
+                printError(
+                    "Problem with retrieving source code of '%s' at %s"
+                    % (self.getFunctionSuperQualifiedName(), self.source_ref)
+                )
+                raise
+
+        return self.function_source
+
+    def getFunctionSuperQualifiedName(self):
+        return "%s.%s" % (
+            self.getParentModule().getFullName(),
+            self.getFunctionBody().getFunctionQualname(),
+        )
+
 
 def makeExpressionFunctionCall(function, values, source_ref):
     assert function.isExpressionFunctionCreation()
@@ -1252,3 +1279,19 @@ class ExpressionFunctionCall(ChildrenHavingFunctionValuesTupleMixin, ExpressionB
 
     def getClosureVariableVersions(self):
         return self.variable_closure_traces
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
