@@ -2,18 +2,33 @@
 
 
 # We are not avoiding these in generated code at all
-# pylint: disable=I0021,too-many-lines
-# pylint: disable=I0021,line-too-long
-# pylint: disable=I0021,too-many-instance-attributes
-# pylint: disable=I0021,too-many-return-statements
+# pylint: disable=I0021,line-too-long,too-many-instance-attributes,too-many-lines
+# pylint: disable=I0021,too-many-arguments,too-many-return-statements,too-many-statements
 
 
 """Hard import nodes
 
 WARNING, this code is GENERATED. Modify the template HardImportReferenceNode.py.j2 instead!
 
-spell-checker: ignore append capitalize casefold center clear copy count decode encode endswith expandtabs extend find format formatmap fromkeys get haskey index insert isalnum isalpha isascii isdecimal isdigit isidentifier islower isnumeric isprintable isspace istitle isupper items iteritems iterkeys itervalues join keys ljust lower lstrip maketrans partition pop popitem prepare remove replace reverse rfind rindex rjust rpartition rsplit rstrip setdefault sort split splitlines startswith strip swapcase title translate update upper values viewitems viewkeys viewvalues zfill
-spell-checker: ignore args chars count default delete encoding end errors fillchar index item iterable keepends key kwargs maxsplit new old pairs prefix sep start stop sub suffix table tabsize value width
+spell-checker: ignore __prepare__ append args autograph buffering capitalize casefold
+spell-checker: ignore center chars clear closefd copy count decode default delete dist
+spell-checker: ignore distribution_name encode encoding end endswith errors exit_code
+spell-checker: ignore expandtabs experimental_attributes experimental_autograph_options
+spell-checker: ignore experimental_compile experimental_follow_type_hints
+spell-checker: ignore experimental_implements experimental_relax_shapes extend file
+spell-checker: ignore fillchar find format format_map formatmap fromkeys func get group
+spell-checker: ignore handle has_key haskey index input_signature insert isalnum isalpha
+spell-checker: ignore isascii isdecimal isdigit isidentifier islower isnumeric isprintable
+spell-checker: ignore isspace istitle isupper item items iterable iteritems iterkeys
+spell-checker: ignore itervalues jit_compile join keepends key keys kwargs ljust lower
+spell-checker: ignore lstrip maketrans maxsplit mode name new newline old opener p package
+spell-checker: ignore package_or_requirement pairs partition path pop popitem prefix
+spell-checker: ignore prepare reduce_retracing remove replace resource resource_name
+spell-checker: ignore reverse rfind rindex rjust rpartition rsplit rstrip s sep setdefault
+spell-checker: ignore sort split splitlines start startswith stop strip sub suffix
+spell-checker: ignore swapcase table tabsize title translate update upper use_errno
+spell-checker: ignore use_last_error value values viewitems viewkeys viewvalues width
+spell-checker: ignore winmode zfill
 """
 import os
 from abc import abstractmethod
@@ -56,6 +71,7 @@ from nuitka.specs.HardImportSpecs import (
     pkg_resources_resource_string_spec,
     pkgutil_get_data_spec,
     sys_exit_spec,
+    tensorflow_function_spec,
 )
 
 from .ChildrenHavingMixins import (
@@ -70,9 +86,10 @@ from .ChildrenHavingMixins import (
     ChildHavingRequirementsTupleMixin,
     ChildHavingSMixin,
     ChildrenHavingFileModeOptionalBufferingOptionalEncodingOptionalErrorsOptionalNewlineOptionalClosefdOptionalOpenerOptionalMixin,
+    ChildrenHavingFuncOptionalInputSignatureOptionalAutographOptionalJitCompileOptionalReduceRetracingOptionalExperimentalImplementsOptionalExperimentalAutographOptionsOptionalExperimentalAttributesOptionalExperimentalRelaxShapesOptionalExperimentalCompileOptionalExperimentalFollowTypeHintsOptionalMixin,
     ChildrenHavingGroupNameOptionalMixin,
-    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalMixin,
-    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalWinmodeOptionalMixin,
+    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLastErrorOptionalMixin,
+    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLastErrorOptionalWinmodeOptionalMixin,
     ChildrenHavingPackageOrRequirementResourceNameMixin,
     ChildrenHavingPackageResourceEncodingOptionalErrorsOptionalMixin,
     ChildrenHavingPackageResourceMixin,
@@ -162,6 +179,7 @@ class ExpressionBuiltinsOpenCallBase(
         opener,
         source_ref,
     ):
+
         ChildrenHavingFileModeOptionalBufferingOptionalEncodingOptionalErrorsOptionalNewlineOptionalClosefdOptionalOpenerOptionalMixin.__init__(
             self,
             file=file,
@@ -263,7 +281,7 @@ hard_import_node_classes[ExpressionCtypesCdllRef] = ctypes_cdll_since_38_spec
 
 
 class ExpressionCtypesCdllSince38CallBase(
-    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalWinmodeOptionalMixin,
+    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLastErrorOptionalWinmodeOptionalMixin,
     ExpressionBase,
 ):
     """Base class for CtypesCdllCall
@@ -278,7 +296,7 @@ class ExpressionCtypesCdllSince38CallBase(
         "mode|optional",
         "handle|optional",
         "use_errno|optional",
-        "use_lasterror|optional",
+        "use_last_error|optional",
         "winmode|optional",
     )
 
@@ -287,15 +305,16 @@ class ExpressionCtypesCdllSince38CallBase(
     spec = ctypes_cdll_since_38_spec
 
     def __init__(
-        self, name, mode, handle, use_errno, use_lasterror, winmode, source_ref
+        self, name, mode, handle, use_errno, use_last_error, winmode, source_ref
     ):
-        ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalWinmodeOptionalMixin.__init__(
+
+        ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLastErrorOptionalWinmodeOptionalMixin.__init__(
             self,
             name=name,
             mode=mode,
             handle=handle,
             use_errno=use_errno,
-            use_lasterror=use_lasterror,
+            use_last_error=use_last_error,
             winmode=winmode,
         )
 
@@ -310,7 +329,7 @@ class ExpressionCtypesCdllSince38CallBase(
                 self.subnode_mode,
                 self.subnode_handle,
                 self.subnode_use_errno,
-                self.subnode_use_lasterror,
+                self.subnode_use_last_error,
                 self.subnode_winmode,
             )
         ):
@@ -333,7 +352,7 @@ class ExpressionCtypesCdllSince38CallBase(
 
 
 class ExpressionCtypesCdllBefore38CallBase(
-    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalMixin,
+    ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLastErrorOptionalMixin,
     ExpressionBase,
 ):
     """Base class for CtypesCdllCall
@@ -348,21 +367,22 @@ class ExpressionCtypesCdllBefore38CallBase(
         "mode|optional",
         "handle|optional",
         "use_errno|optional",
-        "use_lasterror|optional",
+        "use_last_error|optional",
     )
 
     __slots__ = ("attempted",)
 
     spec = ctypes_cdll_before_38_spec
 
-    def __init__(self, name, mode, handle, use_errno, use_lasterror, source_ref):
-        ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalMixin.__init__(
+    def __init__(self, name, mode, handle, use_errno, use_last_error, source_ref):
+
+        ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLastErrorOptionalMixin.__init__(
             self,
             name=name,
             mode=mode,
             handle=handle,
             use_errno=use_errno,
-            use_lasterror=use_lasterror,
+            use_last_error=use_last_error,
         )
 
         ExpressionBase.__init__(self, source_ref)
@@ -376,7 +396,7 @@ class ExpressionCtypesCdllBefore38CallBase(
                 self.subnode_mode,
                 self.subnode_handle,
                 self.subnode_use_errno,
-                self.subnode_use_lasterror,
+                self.subnode_use_last_error,
             )
         ):
             trace_collection.onExceptionRaiseExit(BaseException)
@@ -435,9 +455,9 @@ class ExpressionImportlibMetadataBackportDistributionRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataBackportDistributionRef
-] = importlib_metadata_backport_distribution_spec
+hard_import_node_classes[ExpressionImportlibMetadataBackportDistributionRef] = (
+    importlib_metadata_backport_distribution_spec
+)
 
 
 class ExpressionImportlibMetadataBackportDistributionCallBase(
@@ -455,6 +475,7 @@ class ExpressionImportlibMetadataBackportDistributionCallBase(
     spec = importlib_metadata_backport_distribution_spec
 
     def __init__(self, distribution_name, source_ref):
+
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -528,9 +549,9 @@ class ExpressionImportlibMetadataBackportEntryPointsRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataBackportEntryPointsRef
-] = importlib_metadata_backport_entry_points_spec
+hard_import_node_classes[ExpressionImportlibMetadataBackportEntryPointsRef] = (
+    importlib_metadata_backport_entry_points_spec
+)
 
 
 class ExpressionImportlibMetadataBackportEntryPointsCallBase(
@@ -548,6 +569,7 @@ class ExpressionImportlibMetadataBackportEntryPointsCallBase(
     spec = importlib_metadata_backport_entry_points_spec
 
     def __init__(self, params, source_ref):
+
         ChildHavingParamsTupleMixin.__init__(
             self,
             params=params,
@@ -621,9 +643,9 @@ class ExpressionImportlibMetadataBackportMetadataRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataBackportMetadataRef
-] = importlib_metadata_backport_metadata_spec
+hard_import_node_classes[ExpressionImportlibMetadataBackportMetadataRef] = (
+    importlib_metadata_backport_metadata_spec
+)
 
 
 class ExpressionImportlibMetadataBackportMetadataCallBase(
@@ -641,6 +663,7 @@ class ExpressionImportlibMetadataBackportMetadataCallBase(
     spec = importlib_metadata_backport_metadata_spec
 
     def __init__(self, distribution_name, source_ref):
+
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -714,9 +737,9 @@ class ExpressionImportlibMetadataBackportVersionRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataBackportVersionRef
-] = importlib_metadata_backport_version_spec
+hard_import_node_classes[ExpressionImportlibMetadataBackportVersionRef] = (
+    importlib_metadata_backport_version_spec
+)
 
 
 class ExpressionImportlibMetadataBackportVersionCallBase(
@@ -734,6 +757,7 @@ class ExpressionImportlibMetadataBackportVersionCallBase(
     spec = importlib_metadata_backport_version_spec
 
     def __init__(self, distribution_name, source_ref):
+
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -807,9 +831,9 @@ class ExpressionImportlibMetadataDistributionRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataDistributionRef
-] = importlib_metadata_distribution_spec
+hard_import_node_classes[ExpressionImportlibMetadataDistributionRef] = (
+    importlib_metadata_distribution_spec
+)
 
 
 class ExpressionImportlibMetadataDistributionCallBase(
@@ -827,6 +851,7 @@ class ExpressionImportlibMetadataDistributionCallBase(
     spec = importlib_metadata_distribution_spec
 
     def __init__(self, distribution_name, source_ref):
+
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -917,9 +942,9 @@ class ExpressionImportlibMetadataEntryPointsRef(
             )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataEntryPointsRef
-] = importlib_metadata_entry_points_since_310_spec
+hard_import_node_classes[ExpressionImportlibMetadataEntryPointsRef] = (
+    importlib_metadata_entry_points_since_310_spec
+)
 
 
 class ExpressionImportlibMetadataEntryPointsSince310CallBase(
@@ -939,6 +964,7 @@ class ExpressionImportlibMetadataEntryPointsSince310CallBase(
     spec = importlib_metadata_entry_points_since_310_spec
 
     def __init__(self, params, source_ref):
+
         ChildHavingParamsTupleMixin.__init__(
             self,
             params=params,
@@ -988,6 +1014,7 @@ class ExpressionImportlibMetadataEntryPointsBefore310CallBase(
     spec = importlib_metadata_entry_points_before_310_spec
 
     def __init__(self, source_ref):
+
         ExpressionBase.__init__(self, source_ref)
 
         self.attempted = False
@@ -1053,9 +1080,9 @@ class ExpressionImportlibMetadataMetadataRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataMetadataRef
-] = importlib_metadata_metadata_spec
+hard_import_node_classes[ExpressionImportlibMetadataMetadataRef] = (
+    importlib_metadata_metadata_spec
+)
 
 
 class ExpressionImportlibMetadataMetadataCallBase(
@@ -1073,6 +1100,7 @@ class ExpressionImportlibMetadataMetadataCallBase(
     spec = importlib_metadata_metadata_spec
 
     def __init__(self, distribution_name, source_ref):
+
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -1145,9 +1173,9 @@ class ExpressionImportlibMetadataVersionRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibMetadataVersionRef
-] = importlib_metadata_version_spec
+hard_import_node_classes[ExpressionImportlibMetadataVersionRef] = (
+    importlib_metadata_version_spec
+)
 
 
 class ExpressionImportlibMetadataVersionCallBase(
@@ -1165,6 +1193,7 @@ class ExpressionImportlibMetadataVersionCallBase(
     spec = importlib_metadata_version_spec
 
     def __init__(self, distribution_name, source_ref):
+
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -1237,9 +1266,9 @@ class ExpressionImportlibResourcesBackportFilesRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibResourcesBackportFilesRef
-] = importlib_resources_backport_files_spec
+hard_import_node_classes[ExpressionImportlibResourcesBackportFilesRef] = (
+    importlib_resources_backport_files_spec
+)
 
 
 class ExpressionImportlibResourcesBackportFilesCallBase(
@@ -1257,6 +1286,7 @@ class ExpressionImportlibResourcesBackportFilesCallBase(
     spec = importlib_resources_backport_files_spec
 
     def __init__(self, package, source_ref):
+
         ChildHavingPackageMixin.__init__(
             self,
             package=package,
@@ -1330,9 +1360,9 @@ class ExpressionImportlibResourcesBackportReadBinaryRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibResourcesBackportReadBinaryRef
-] = importlib_resources_backport_read_binary_spec
+hard_import_node_classes[ExpressionImportlibResourcesBackportReadBinaryRef] = (
+    importlib_resources_backport_read_binary_spec
+)
 
 
 class ExpressionImportlibResourcesBackportReadBinaryCallBase(
@@ -1353,6 +1383,7 @@ class ExpressionImportlibResourcesBackportReadBinaryCallBase(
     spec = importlib_resources_backport_read_binary_spec
 
     def __init__(self, package, resource, source_ref):
+
         ChildrenHavingPackageResourceMixin.__init__(
             self,
             package=package,
@@ -1430,9 +1461,9 @@ class ExpressionImportlibResourcesBackportReadTextRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibResourcesBackportReadTextRef
-] = importlib_resources_backport_read_text_spec
+hard_import_node_classes[ExpressionImportlibResourcesBackportReadTextRef] = (
+    importlib_resources_backport_read_text_spec
+)
 
 
 class ExpressionImportlibResourcesBackportReadTextCallBase(
@@ -1457,6 +1488,7 @@ class ExpressionImportlibResourcesBackportReadTextCallBase(
     spec = importlib_resources_backport_read_text_spec
 
     def __init__(self, package, resource, encoding, errors, source_ref):
+
         ChildrenHavingPackageResourceEncodingOptionalErrorsOptionalMixin.__init__(
             self,
             package=package,
@@ -1536,9 +1568,9 @@ class ExpressionImportlibResourcesFilesRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibResourcesFilesRef
-] = importlib_resources_files_spec
+hard_import_node_classes[ExpressionImportlibResourcesFilesRef] = (
+    importlib_resources_files_spec
+)
 
 
 class ExpressionImportlibResourcesFilesCallBase(
@@ -1556,6 +1588,7 @@ class ExpressionImportlibResourcesFilesCallBase(
     spec = importlib_resources_files_spec
 
     def __init__(self, package, source_ref):
+
         ChildHavingPackageMixin.__init__(
             self,
             package=package,
@@ -1628,9 +1661,9 @@ class ExpressionImportlibResourcesReadBinaryRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibResourcesReadBinaryRef
-] = importlib_resources_read_binary_spec
+hard_import_node_classes[ExpressionImportlibResourcesReadBinaryRef] = (
+    importlib_resources_read_binary_spec
+)
 
 
 class ExpressionImportlibResourcesReadBinaryCallBase(
@@ -1651,6 +1684,7 @@ class ExpressionImportlibResourcesReadBinaryCallBase(
     spec = importlib_resources_read_binary_spec
 
     def __init__(self, package, resource, source_ref):
+
         ChildrenHavingPackageResourceMixin.__init__(
             self,
             package=package,
@@ -1727,9 +1761,9 @@ class ExpressionImportlibResourcesReadTextRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionImportlibResourcesReadTextRef
-] = importlib_resources_read_text_spec
+hard_import_node_classes[ExpressionImportlibResourcesReadTextRef] = (
+    importlib_resources_read_text_spec
+)
 
 
 class ExpressionImportlibResourcesReadTextCallBase(
@@ -1754,6 +1788,7 @@ class ExpressionImportlibResourcesReadTextCallBase(
     spec = importlib_resources_read_text_spec
 
     def __init__(self, package, resource, encoding, errors, source_ref):
+
         ChildrenHavingPackageResourceEncodingOptionalErrorsOptionalMixin.__init__(
             self,
             package=package,
@@ -1846,6 +1881,7 @@ class ExpressionOsListdirCallBase(ChildHavingPathOptionalMixin, ExpressionBase):
     spec = os_listdir_spec
 
     def __init__(self, path, source_ref):
+
         ChildHavingPathOptionalMixin.__init__(
             self,
             path=path,
@@ -1927,6 +1963,7 @@ class ExpressionOsPathAbspathCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_abspath_spec
 
     def __init__(self, path, source_ref):
+
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -2008,6 +2045,7 @@ class ExpressionOsPathBasenameCallBase(ChildHavingPMixin, ExpressionBase):
     spec = os_path_basename_spec
 
     def __init__(self, p, source_ref):
+
         ChildHavingPMixin.__init__(
             self,
             p=p,
@@ -2089,6 +2127,7 @@ class ExpressionOsPathDirnameCallBase(ChildHavingPMixin, ExpressionBase):
     spec = os_path_dirname_spec
 
     def __init__(self, p, source_ref):
+
         ChildHavingPMixin.__init__(
             self,
             p=p,
@@ -2170,6 +2209,7 @@ class ExpressionOsPathExistsCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_exists_spec
 
     def __init__(self, path, source_ref):
+
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -2253,6 +2293,7 @@ class ExpressionOsPathIsabsCallBase(
     spec = os_path_isabs_spec
 
     def __init__(self, s, source_ref):
+
         ChildHavingSMixin.__init__(
             self,
             s=s,
@@ -2334,6 +2375,7 @@ class ExpressionOsPathIsdirCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_isdir_spec
 
     def __init__(self, path, source_ref):
+
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -2415,6 +2457,7 @@ class ExpressionOsPathIsfileCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_isfile_spec
 
     def __init__(self, path, source_ref):
+
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -2494,6 +2537,7 @@ class ExpressionOsUnameCallBase(ExpressionBase):
     spec = os_uname_spec
 
     def __init__(self, source_ref):
+
         ExpressionBase.__init__(self, source_ref)
 
         self.attempted = False
@@ -2559,9 +2603,9 @@ class ExpressionPkgResourcesGetDistributionRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionPkgResourcesGetDistributionRef
-] = pkg_resources_get_distribution_spec
+hard_import_node_classes[ExpressionPkgResourcesGetDistributionRef] = (
+    pkg_resources_get_distribution_spec
+)
 
 
 class ExpressionPkgResourcesGetDistributionCallBase(
@@ -2579,6 +2623,7 @@ class ExpressionPkgResourcesGetDistributionCallBase(
     spec = pkg_resources_get_distribution_spec
 
     def __init__(self, dist, source_ref):
+
         ChildHavingDistMixin.__init__(
             self,
             dist=dist,
@@ -2652,9 +2697,9 @@ class ExpressionPkgResourcesIterEntryPointsRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionPkgResourcesIterEntryPointsRef
-] = pkg_resources_iter_entry_points_spec
+hard_import_node_classes[ExpressionPkgResourcesIterEntryPointsRef] = (
+    pkg_resources_iter_entry_points_spec
+)
 
 
 class ExpressionPkgResourcesIterEntryPointsCallBase(
@@ -2675,6 +2720,7 @@ class ExpressionPkgResourcesIterEntryPointsCallBase(
     spec = pkg_resources_iter_entry_points_spec
 
     def __init__(self, group, name, source_ref):
+
         ChildrenHavingGroupNameOptionalMixin.__init__(
             self,
             group=group,
@@ -2768,6 +2814,7 @@ class ExpressionPkgResourcesRequireCallBase(
     spec = pkg_resources_require_spec
 
     def __init__(self, requirements, source_ref):
+
         ChildHavingRequirementsTupleMixin.__init__(
             self,
             requirements=requirements,
@@ -2838,9 +2885,9 @@ class ExpressionPkgResourcesResourceStreamRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionPkgResourcesResourceStreamRef
-] = pkg_resources_resource_stream_spec
+hard_import_node_classes[ExpressionPkgResourcesResourceStreamRef] = (
+    pkg_resources_resource_stream_spec
+)
 
 
 class ExpressionPkgResourcesResourceStreamCallBase(
@@ -2861,6 +2908,7 @@ class ExpressionPkgResourcesResourceStreamCallBase(
     spec = pkg_resources_resource_stream_spec
 
     def __init__(self, package_or_requirement, resource_name, source_ref):
+
         ChildrenHavingPackageOrRequirementResourceNameMixin.__init__(
             self,
             package_or_requirement=package_or_requirement,
@@ -2938,9 +2986,9 @@ class ExpressionPkgResourcesResourceStringRef(
         )
 
 
-hard_import_node_classes[
-    ExpressionPkgResourcesResourceStringRef
-] = pkg_resources_resource_string_spec
+hard_import_node_classes[ExpressionPkgResourcesResourceStringRef] = (
+    pkg_resources_resource_string_spec
+)
 
 
 class ExpressionPkgResourcesResourceStringCallBase(
@@ -2961,6 +3009,7 @@ class ExpressionPkgResourcesResourceStringCallBase(
     spec = pkg_resources_resource_string_spec
 
     def __init__(self, package_or_requirement, resource_name, source_ref):
+
         ChildrenHavingPackageOrRequirementResourceNameMixin.__init__(
             self,
             package_or_requirement=package_or_requirement,
@@ -3055,6 +3104,7 @@ class ExpressionPkgutilGetDataCallBase(
     spec = pkgutil_get_data_spec
 
     def __init__(self, package, resource, source_ref):
+
         ChildrenHavingPackageResourceMixin.__init__(
             self,
             package=package,
@@ -3140,6 +3190,7 @@ class ExpressionSysExitCallBase(ChildHavingExitCodeOptionalMixin, ExpressionBase
     spec = sys_exit_spec
 
     def __init__(self, exit_code, source_ref):
+
         ChildHavingExitCodeOptionalMixin.__init__(
             self,
             exit_code=exit_code,
@@ -3172,6 +3223,140 @@ class ExpressionSysExitCallBase(ChildHavingExitCodeOptionalMixin, ExpressionBase
         return True
 
 
+class ExpressionTensorflowFunctionRef(ExpressionImportModuleNameHardExistsSpecificBase):
+    """Function reference tensorflow.function"""
+
+    kind = "EXPRESSION_TENSORFLOW_FUNCTION_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name="tensorflow",
+            import_name="function",
+            module_guaranteed=not shallMakeModule(),
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .TensorflowNodes import ExpressionTensorflowFunctionCall
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=ExpressionTensorflowFunctionCall,
+            builtin_spec=tensorflow_function_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'tensorflow.function' recognized.",
+        )
+
+
+hard_import_node_classes[ExpressionTensorflowFunctionRef] = tensorflow_function_spec
+
+
+class ExpressionTensorflowFunctionCallBase(
+    ChildrenHavingFuncOptionalInputSignatureOptionalAutographOptionalJitCompileOptionalReduceRetracingOptionalExperimentalImplementsOptionalExperimentalAutographOptionsOptionalExperimentalAttributesOptionalExperimentalRelaxShapesOptionalExperimentalCompileOptionalExperimentalFollowTypeHintsOptionalMixin,
+    ExpressionBase,
+):
+    """Base class for TensorflowFunctionCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = (
+        "func|optional",
+        "input_signature|optional",
+        "autograph|optional",
+        "jit_compile|optional",
+        "reduce_retracing|optional",
+        "experimental_implements|optional",
+        "experimental_autograph_options|optional",
+        "experimental_attributes|optional",
+        "experimental_relax_shapes|optional",
+        "experimental_compile|optional",
+        "experimental_follow_type_hints|optional",
+    )
+
+    __slots__ = ("attempted",)
+
+    spec = tensorflow_function_spec
+
+    def __init__(
+        self,
+        func,
+        input_signature,
+        autograph,
+        jit_compile,
+        reduce_retracing,
+        experimental_implements,
+        experimental_autograph_options,
+        experimental_attributes,
+        experimental_relax_shapes,
+        experimental_compile,
+        experimental_follow_type_hints,
+        source_ref,
+    ):
+
+        ChildrenHavingFuncOptionalInputSignatureOptionalAutographOptionalJitCompileOptionalReduceRetracingOptionalExperimentalImplementsOptionalExperimentalAutographOptionsOptionalExperimentalAttributesOptionalExperimentalRelaxShapesOptionalExperimentalCompileOptionalExperimentalFollowTypeHintsOptionalMixin.__init__(
+            self,
+            func=func,
+            input_signature=input_signature,
+            autograph=autograph,
+            jit_compile=jit_compile,
+            reduce_retracing=reduce_retracing,
+            experimental_implements=experimental_implements,
+            experimental_autograph_options=experimental_autograph_options,
+            experimental_attributes=experimental_attributes,
+            experimental_relax_shapes=experimental_relax_shapes,
+            experimental_compile=experimental_compile,
+            experimental_follow_type_hints=experimental_follow_type_hints,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+        # In module mode, we expect a changing environment, cannot optimize this
+        self.attempted = shallMakeModule()
+
+    def computeExpression(self, trace_collection):
+        if self.attempted or not tensorflow_function_spec.isCompileTimeComputable(
+            (
+                self.subnode_func,
+                self.subnode_input_signature,
+                self.subnode_autograph,
+                self.subnode_jit_compile,
+                self.subnode_reduce_retracing,
+                self.subnode_experimental_implements,
+                self.subnode_experimental_autograph_options,
+                self.subnode_experimental_attributes,
+                self.subnode_experimental_relax_shapes,
+                self.subnode_experimental_compile,
+                self.subnode_experimental_follow_type_hints,
+            )
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
