@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Control the flow of optimizations applied to node tree.
 
 Applies abstract execution on all so far known modules until no more
@@ -22,7 +8,6 @@ optimization is possible. Every successful optimization to anything might
 make others possible.
 """
 
-
 import inspect
 
 from nuitka import ModuleRegistry, Options, Variables
@@ -66,7 +51,7 @@ def signalChange(tags, source_ref, message):
 
 
 def optimizeCompiledPythonModule(module):
-    optimization_logger.info_fileoutput(
+    optimization_logger.info_if_file(
         "Doing module local optimizations for '{module_name}'.".format(
             module_name=module.getFullName()
         ),
@@ -101,29 +86,25 @@ def optimizeCompiledPythonModule(module):
 
         Graphs.onModuleOptimizationStep(module)
 
-        # Ignore other modules brought into the game.
-        if "new_code" in tag_set:
-            tag_set.remove("new_code")
-
         # Search for local change tags.
         if not tag_set:
             unchanged_count += 1
 
             if unchanged_count == 1 and pass_count == 1:
-                optimization_logger.info_fileoutput(
+                optimization_logger.info_if_file(
                     "Not changed, but retrying one more time.",
                     other_logger=progress_logger,
                 )
                 continue
 
-            optimization_logger.info_fileoutput(
+            optimization_logger.info_if_file(
                 "Finished with the module.", other_logger=progress_logger
             )
             break
 
         unchanged_count = 0
 
-        optimization_logger.info_fileoutput(
+        optimization_logger.info_if_file(
             "Not finished with the module due to following change kinds: %s"
             % ",".join(sorted(tag_set)),
             other_logger=progress_logger,
@@ -137,7 +118,7 @@ def optimizeCompiledPythonModule(module):
             "Memory usage changed during optimization of '%s'" % (module.getFullName())
         )
 
-    considerUsedModules(module=module, signal_change=signalChange)
+    considerUsedModules(module=module, pass_count=pass_count)
 
     return touched
 
@@ -153,16 +134,16 @@ def optimizeUncompiledPythonModule(module):
     # Pick up parent package if any.
     module.attemptRecursion()
 
-    considerUsedModules(module=module, signal_change=signalChange)
+    considerUsedModules(module=module, pass_count=pass_count)
 
-    Plugins.considerImplicitImports(module=module, signal_change=signalChange)
+    Plugins.considerImplicitImports(module=module)
 
 
 def optimizeExtensionModule(module):
     # Pick up parent package if any.
     module.attemptRecursion()
 
-    Plugins.considerImplicitImports(module=module, signal_change=signalChange)
+    Plugins.considerImplicitImports(module=module)
 
 
 def optimizeModule(module):
@@ -195,7 +176,7 @@ def _restartProgress():
     closeProgressBar()
     pass_count += 1
 
-    optimization_logger.info_fileoutput(
+    optimization_logger.info_if_file(
         "PASS %d:" % pass_count, other_logger=progress_logger
     )
 
@@ -210,7 +191,7 @@ def _restartProgress():
 
 
 def _traceProgressModuleStart(current_module):
-    optimization_logger.info_fileoutput(
+    optimization_logger.info_if_file(
         """\
 Optimizing module '{module_name}', {remaining:d} more modules to go \
 after that.""".format(
@@ -231,11 +212,13 @@ after that.""".format(
         reportMemoryUsage(
             "optimization/%d/%s" % (pass_count, current_module.getFullName()),
             (
-                "Total memory usage before optimizing module '%s'"
-                % current_module.getFullName()
-            )
-            if Options.isShowProgress() or Options.isShowMemory()
-            else None,
+                (
+                    "Total memory usage before optimizing module '%s'"
+                    % current_module.getFullName()
+                )
+                if Options.isShowProgress() or Options.isShowMemory()
+                else None
+            ),
         )
 
 
@@ -275,14 +258,25 @@ def makeOptimizationPass():
 
     _restartProgress()
 
+    main_module = None
+    stdlib_phase_done = False
+
     while True:
         current_module = ModuleRegistry.nextModule()
 
         if current_module is None:
-            # TODO: Internal module seems to cause extra passes.
-            # optimizeModule(getInternalModule())
+            if main_module is not None and pass_count == 1:
+                considerUsedModules(module=main_module, pass_count=-1)
+
+                stdlib_phase_done = True
+                main_module = None
+                continue
+
             break
 
+        if current_module.isMainModule() and not stdlib_phase_done:
+            main_module = current_module
+
         _traceProgressModuleStart(current_module)
 
         module_name = current_module.getFullName()
@@ -350,3 +344,19 @@ def optimizeModules(output_filename):
         finished = makeOptimizationPass()
 
     Graphs.endGraph(output_filename)
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
