@@ -17,6 +17,7 @@ import functools
 import inspect
 import os
 import sys
+import unittest
 
 from nuitka import Options
 from nuitka.__past__ import iter_modules, unicode
@@ -135,6 +136,7 @@ def _getEvaluationContext():
             "deployment": isDeploymentMode(),
             # Querying package versions.
             "version": _getPackageVersion,
+            "version_str": _getPackageVersionStr,
             "get_dist_name": _getDistributionNameFromPackageName,
             "plugin": _isPluginActive,
             "no_asserts": hasPythonFlagNoAsserts(),
@@ -155,6 +157,7 @@ def _getEvaluationContext():
             "os": os,
             "importlib": _getImportLibModule(),
             "appdirs": getAppdirsModule(),
+            "unittest": unittest,
             # Python version string
             "python_version_str": python_version_str,
             "python_version_full_str": python_version_full_str,
@@ -184,9 +187,9 @@ def _getEvaluationContext():
             numeric_version = int(big) * 256 + int(major) * 16
             is_same_or_higher_version = python_version >= numeric_version
 
-            _context_dict[
-                "python" + big + major + "_or_higher"
-            ] = is_same_or_higher_version
+            _context_dict["python" + big + major + "_or_higher"] = (
+                is_same_or_higher_version
+            )
             _context_dict["before_python" + big + major] = not is_same_or_higher_version
 
         _context_dict["before_python3"] = python_version < 0x300
@@ -207,6 +210,8 @@ def _convertVersionToTuple(version_str):
 
 
 def _getPackageNameFromDistributionName(distribution_name):
+    # spell-checker: ignore opencv, pyobjc, objc
+
     if distribution_name in ("opencv-python", "opencv-python-headless"):
         return "cv2"
     elif distribution_name == "pyobjc":
@@ -270,6 +275,15 @@ def _getPackageVersion(distribution_name):
     return _package_versions[distribution_name]
 
 
+def _getPackageVersionStr(distribution_name):
+    version = _getPackageVersion(distribution_name)
+
+    if version is not None:
+        version = ".".join(str(d) for d in version)
+
+    return version
+
+
 def _getModuleDirectory(module_name):
     from nuitka.importing.Importing import locateModule
 
@@ -290,7 +304,7 @@ def _iterate_module_names(package_name):
         module_name = package_name.getChildNamed(module_info.name)
         result.append(module_name.asString())
 
-        if module_info.ispkg:
+        if module_info.ispkg:  # spell-checker: ignore ispkg
             result.extend(_iterate_module_names(package_name=module_name))
 
     return result
@@ -1663,6 +1677,26 @@ class NuitkaYamlPluginBase(NuitkaPluginBase):
                     if decide_relevant(key, value):
                         yield key, value
 
+    def getYamlConfigItemSet(
+        self, module_name, section, item_name, decide_relevant, recursive
+    ):
+        for item_config in self.getYamlConfigItem(
+            module_name=module_name,
+            section=section,
+            item_name=item_name,
+            decide_relevant=None,
+            default=(),
+            recursive=recursive,
+        ):
+            if recursive:
+                for value in item_config[1]:
+                    if decide_relevant is None or decide_relevant(value):
+                        yield item_config[0], value
+            else:
+                for value in item_config:
+                    if decide_relevant is None or decide_relevant(value):
+                        yield value
+
 
 def standalone_only(func):
     """For plugins that have functionality that should be done in standalone mode only."""
