@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Standard plug-in to tell user about needed or useful options for packages.
 
 When certain GUI packages are used, disabling the console may or may not be what
@@ -29,21 +15,17 @@ from nuitka.Options import (
     shallCreateAppBundle,
     shallDisableConsoleWindow,
 )
-from nuitka.plugins.PluginBase import NuitkaPluginBase
+from nuitka.plugins.PluginBase import NuitkaYamlPluginBase
 from nuitka.utils.Utils import isMacOS
-from nuitka.utils.Yaml import getYamlPackageConfiguration
 
 
-class NuitkaPluginOptionsNanny(NuitkaPluginBase):
+class NuitkaPluginOptionsNanny(NuitkaYamlPluginBase):
     plugin_name = "options-nanny"
 
     plugin_desc = (
         "Inform the user about potential problems as per package configuration files."
     )
 
-    def __init__(self):
-        self.config = getYamlPackageConfiguration()
-
     @staticmethod
     def isAlwaysEnabled():
         return True
@@ -59,26 +41,28 @@ class NuitkaPluginOptionsNanny(NuitkaPluginBase):
         if support_info == "ignore":
             return
 
-        if support_info != "parameter":
+        if support_info == "parameter":
+            message = "Module has parameter: " + description
+        elif support_info == "plugin":
+            message = "Module has plugin consideration: " + description
+        else:
             if condition != "True":
                 problem_desc = (
-                    "incomplete support due to untrue condition '%s'" % condition
+                    " with incomplete support due to condition '%s'" % condition
                 )
             else:
-                problem_desc = "incomplete support"
+                problem_desc = " with incomplete support"
 
-            message = "Using module '%s' (version %s) with %s: %s" % (
+            message = "Using module '%s' (version %s)%s: %s" % (
                 full_name,
                 ".".join(str(d) for d in self.getPackageVersion(full_name)),
                 problem_desc,
                 description,
             )
-        else:
-            message = "Module has parameter: " + description
 
         if support_info == "error":
             self.sysexit(message)
-        elif support_info in ("warning", "parameter"):
+        elif support_info in ("warning", "parameter", "plugin"):
             self.warning(message)
         elif support_info == "info":
             self.info(message)
@@ -187,3 +171,19 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
                         )
 
         return ()
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
