@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Standard plug-in to handle pkg_resource special needs.
 
 Nuitka can detect some things that "pkg_resources" may not even be able to during
@@ -22,10 +8,10 @@ runtime, but that is done by nodes and optimization. But there are other things,
 that need special case, e.g. the registration of the loader class.
 """
 
-
 import re
 
 from nuitka.plugins.PluginBase import NuitkaPluginBase
+from nuitka.PythonVersions import python_version
 from nuitka.utils.Utils import withNoDeprecationWarning
 
 
@@ -43,11 +29,14 @@ class NuitkaPluginResources(NuitkaPluginBase):
                 self.pkg_resources = pkg_resources
 
         try:
-            import importlib_metadata
+            if python_version >= 0x380:
+                from importlib import metadata
+            else:
+                import importlib_metadata as metadata
         except (ImportError, SyntaxError, RuntimeError):
             self.metadata = None
         else:
-            self.metadata = importlib_metadata
+            self.metadata = metadata
 
     @staticmethod
     def isAlwaysEnabled():
@@ -90,7 +79,7 @@ sys.exit(%(module_name)s.%(main_name)s)
             "main_name": main_name,
         }
 
-    def onModuleSourceCode(self, module_name, source_code):
+    def onModuleSourceCode(self, module_name, source_filename, source_code):
         if module_name == "__main__":
             match = re.search(
                 "\n# EASY-INSTALL-ENTRY-SCRIPT: '(.*?)','(.*?)','(.*?)'", source_code
@@ -143,3 +132,19 @@ register_loader_type(__nuitka_loader_type, NuitkaProvider)
             code,
             """Registering Nuitka loader with "pkg_resources".""",
         )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
