@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Standard plug-in to make PyQt and PySide work well in standalone mode.
 
 To run properly, these need the Qt plugins copied along, which have their
@@ -39,18 +25,20 @@ from nuitka.plugins.Plugins import (
 )
 from nuitka.PythonFlavors import isAnacondaPython
 from nuitka.PythonVersions import python_version
+from nuitka.utils.Distributions import (
+    getDistributionFromModuleName,
+    getDistributionInstallerName,
+    getDistributionName,
+)
 from nuitka.utils.FileOperations import getFileList, listDir
 from nuitka.utils.ModuleNames import ModuleName
-from nuitka.utils.Utils import (
-    getArchitecture,
-    isDebianBasedLinux,
-    isLinux,
-    isMacOS,
-    isWin32Windows,
-)
+from nuitka.utils.Utils import getArchitecture, isMacOS, isWin32Windows
 
 
 class NuitkaPluginQtBindingsPluginBase(NuitkaPluginBase):
+    # We are a bit detail rich and caching a bunch,
+    # pylint: disable=too-many-instance-attributes
+
     # Automatically suppress detectors for any other toolkit
     plugin_gui_toolkit = True
 
@@ -59,18 +47,41 @@ class NuitkaPluginQtBindingsPluginBase(NuitkaPluginBase):
 
     warned_about = set()
 
-    def __init__(self, qt_plugins, no_qt_translations):
-        if not qt_plugins:
-            qt_plugins = ["sensible"]
+    def __init__(self, include_qt_plugins, noinclude_qt_plugins, no_qt_translations):
+        self.distribution = getDistributionFromModuleName(self.binding_name)
 
-        qt_plugins = sum([value.split(",") for value in qt_plugins], [])
+        if self.distribution is None:
+            self.sysexit(
+                "Error, failed to locate the %s installation." % self.binding_name
+            )
+
+        self.distribution_name = getDistributionName(self.distribution)
+
+        self.installer_name = getDistributionInstallerName(self.distribution_name)
+
+        # Qt plugin directories found.
+        self.qt_plugins_dirs = None
+
+        sensible_qt_plugins = self._getSensiblePlugins()
+
+        include_qt_plugins = OrderedSet(
+            sum([value.split(",") for value in include_qt_plugins], [])
+        )
+
+        # Useless, but nice for old option usage, where expanding it meant to repeat it.
+        if "sensible" in include_qt_plugins:
+            include_qt_plugins.remove("sensible")
+
+        self.qt_plugins = sensible_qt_plugins
+        self.qt_plugins.update(include_qt_plugins)
+
+        for noinclude_qt_plugin in noinclude_qt_plugins:
+            self.qt_plugins.discard(noinclude_qt_plugin)
 
-        self.qt_plugins = OrderedSet(x.strip().lower() for x in qt_plugins)
         self.no_qt_translations = no_qt_translations
 
         self.web_engine_done_binaries = False
         self.web_engine_done_data = False
-        self.qt_plugins_dirs = None
 
         self.binding_package_name = ModuleName(self.binding_name)
 
@@ -97,15 +108,26 @@ class NuitkaPluginQtBindingsPluginBase(NuitkaPluginBase):
         group.add_option(
             "--include-qt-plugins",
             action="append",
-            dest="qt_plugins",
+            dest="include_qt_plugins",
             default=[],
             help="""\
 Which Qt plugins to include. These can be big with dependencies, so
-by default only the sensible ones are included, but you can also put
+by default only the "sensible" ones are included, but you can also put
 "all" or list them individually. If you specify something that does
 not exist, a list of all available will be given.""",
         )
 
+        group.add_option(
+            "--noinclude-qt-plugins",
+            action="append",
+            dest="noinclude_qt_plugins",
+            default=[],
+            help="""\
+Which Qt plugins to not include. This removes things, so you can
+ask to include "all" and selectively remove from there, or even
+from the default sensible list.""",
+        )
+
         group.add_option(
             "--noinclude-qt-translations",
             action="store_true",
@@ -175,41 +197,35 @@ of files that you may not want to be included.""",
         """Where does the Qt bindings package expect the web process executable."""
         return "."
 
-    def getQtPluginsSelected(self):
-        # Resolve "sensible on first use"
-        if "sensible" in self.qt_plugins:
-            # Most used ones with low dependencies.
-            self.qt_plugins.update(
-                tuple(
-                    family
-                    for family in (
-                        "imageformats",
-                        "iconengines",
-                        "mediaservice",
-                        "printsupport",
-                        "platforms",
-                        "platformthemes",
-                        "styles",
-                        # Wayland on Linux needs these
-                        "wayland-shell-integration",
-                        "wayland-decoration-client",
-                        "wayland-graphics-integration-client",
-                        "egldeviceintegrations",
-                        # OpenGL rendering, maybe should be something separate.
-                        "xcbglintegrations",
-                        # SSL network needs those
-                        "tls",
-                    )
-                    if self.hasPluginFamily(family)
+    def _getSensiblePlugins(self):
+        # spell-checker: ignore imageformats,iconengines,mediaservice,printsupport
+        # spell-checker: ignore platformthemes,egldeviceintegrations,xcbglintegrations
+        return OrderedSet(
+            tuple(
+                family
+                for family in (
+                    "imageformats",
+                    "iconengines",
+                    "mediaservice",
+                    "printsupport",
+                    "platforms",
+                    "platformthemes",
+                    "styles",
+                    # Wayland on Linux needs these
+                    "wayland-shell-integration",
+                    "wayland-decoration-client",
+                    "wayland-graphics-integration-client",
+                    "egldeviceintegrations",
+                    # OpenGL rendering, maybe should be something separate.
+                    "xcbglintegrations",
+                    # SSL network needs those
+                    "tls",
                 )
+                if self.hasPluginFamily(family)
             )
+        )
 
-            self.qt_plugins.remove("sensible")
-
-            # Make sure the above didn't detect nothing, which would be
-            # indicating the check to be bad.
-            assert self.qt_plugins
-
+    def getQtPluginsSelected(self):
         return self.qt_plugins
 
     def hasQtPluginSelected(self, plugin_name):
@@ -375,7 +391,8 @@ import %(binding_name)s.QtCore
     def _getQmlFileList(self, dlls):
         qml_plugin_dir = self._getQmlDirectory()
 
-        # List all file types of the QML plugin folder that are data files and not DLLs.
+        # List all file types of the QML plugin folder that are data files and
+        # not DLLs, spell-checker: ignore qmlc,qmltypes,metainfo,qmldir
         datafile_suffixes = (
             ".qml",
             ".qmlc",
@@ -420,6 +437,7 @@ import %(binding_name)s.QtCore
                         self.getQtPluginTargetPath(),
                         filename_relative,
                     ),
+                    module_name=self.binding_package_name,
                     package_name=self.binding_package_name,
                     reason="qt plugin",
                 )
@@ -658,7 +676,7 @@ system Qt plugins, which may be from another Qt version.""",
         if full_name == self.binding_name and isWin32Windows():
             code = """\
 import os
-path = os.environ.get("PATH", "")
+path = os.getenv("PATH", "")
 if not path.startswith(__nuitka_binary_dir):
     os.environ["PATH"] = __nuitka_binary_dir + ";" + path
 """
@@ -849,6 +867,7 @@ Prefix = .
                     dest_path=os.path.normpath(
                         os.path.join(self._getWebEngineTargetDir(), filename_relative)
                     ),
+                    module_name=full_name,
                     package_name=full_name,
                     reason="needed by '%s'" % full_name.asString(),
                 )
@@ -862,17 +881,6 @@ Prefix = .
 
         self.web_engine_done_binaries = True  # prevent multiple copies
 
-    def decideAllowOutsideDependencies(self, module_name):
-        if isLinux() and module_name.hasNamespace(self.binding_name):
-            if isDebianBasedLinux():
-                module_filename = self.locateModule(self.binding_name)
-
-                return "dist-packages" in module_filename.split("/")
-            else:
-                return False
-
-        return None
-
     def getQtPluginTargetPath(self):
         if self.binding_name == "PyQt6":
             return os.path.join(self.binding_name, "Qt6", "plugins")
@@ -920,10 +928,13 @@ Prefix = .
                 count = 0
                 for filename in qt_bin_files:
                     basename = os.path.basename(filename).lower()
+                    # spell-checker: ignore libeay32,ssleay32
+
                     if basename in ("libeay32.dll", "ssleay32.dll"):
                         yield self.makeDllEntryPoint(
                             source_path=filename,
                             dest_path=basename,
+                            module_name=full_name,
                             package_name=full_name,
                             reason="needed by '%s'" % full_name.asString(),
                         )
@@ -948,6 +959,7 @@ Prefix = .
                             qml_target_dir,
                             filename_relative,
                         ),
+                        module_name=full_name,
                         package_name=full_name,
                         reason="Qt QML plugin DLL",
                     )
@@ -965,6 +977,7 @@ Prefix = .
                             yield self.makeDllEntryPoint(
                                 source_path=filename,
                                 dest_path=basename,
+                                module_name=full_name,
                                 package_name=full_name,
                                 reason="needed by OpenGL for '%s'"
                                 % full_name.asString(),
@@ -993,6 +1006,7 @@ Prefix = .
                     )
 
                 # Manually loaded DLLs by Qt5.
+                # spell-checker: ignore libcrypto
                 for dll_basename in ("libssl-1_1", "libcrypto-1_1"):
                     dll_filename = dll_basename + arch_suffix + ".dll"
 
@@ -1003,6 +1017,7 @@ Prefix = .
                             yield self.makeDllEntryPoint(
                                 source_path=candidate,
                                 dest_path=dll_filename,
+                                module_name=full_name,
                                 package_name=full_name,
                                 reason="needed by '%s'" % full_name.asString(),
                             )
@@ -1015,6 +1030,7 @@ Prefix = .
                 yield self.makeDllEntryPoint(
                     source_path=dll_path,
                     dest_path=os.path.basename(dll_path),
+                    module_name=full_name,
                     package_name=full_name,
                     reason="needed by '%s'" % full_name.asString(),
                 )
@@ -1024,38 +1040,11 @@ Prefix = .
                 yield self.makeDllEntryPoint(
                     source_path=dll_path,
                     dest_path=os.path.basename(dll_path),
+                    module_name=full_name,
                     package_name=full_name,
                     reason="needed by '%s'" % full_name.asString(),
                 )
 
-    def removeDllDependencies(self, dll_filename, dll_filenames):
-        for value in self.getQtPluginDirs():
-            # TODO: That is not a proper check if a file is below that.
-            if dll_filename.startswith(value):
-                for sub_dll_filename in dll_filenames:
-                    for badword in (
-                        "libKF5",
-                        "libkfontinst",
-                        "libkorganizer",
-                        "libplasma",
-                        "libakregator",
-                        "libdolphin",
-                        "libnoteshared",
-                        "libknotes",
-                        "libsystemsettings",
-                        "libkerfuffle",
-                        "libkaddressbook",
-                        "libkworkspace",
-                        "libkmail",
-                        "libmilou",
-                        "libtaskmanager",
-                        "libkonsole",
-                        "libgwenview",
-                        "libweather_ion",
-                    ):
-                        if os.path.basename(sub_dll_filename).startswith(badword):
-                            yield sub_dll_filename
-
     def onModuleEncounter(
         self, using_module_name, module_name, module_filename, module_kind
     ):
@@ -1101,8 +1090,9 @@ behavior with the uncompiled code."""
             module_set=module_set, plugin_binding_name=self.binding_name
         )
 
-    def onModuleSourceCode(self, module_name, source_code):
+    def onModuleSourceCode(self, module_name, source_filename, source_code):
         """Third party packages that make binding selections."""
+        # spell-checker: ignore pyqtgraph
         if module_name.hasNamespace("pyqtgraph"):
             # TODO: Add a mechanism to force all variable references of a name to something
             # during tree building, that would cover all uses in a nicer way.
@@ -1115,6 +1105,17 @@ behavior with the uncompiled code."""
 
         return source_code
 
+    def onDataFileTags(self, included_datafile):
+        if included_datafile.dest_path.endswith(
+            ".qml"
+        ) and not self.hasQtPluginSelected("qml"):
+            self.warning(
+                """Including QML file %s, but not having Qt qml plugins is unlikely \
+to work. Consider using '--include-qt-plugins=qml' to include the \
+necessary files to use it."""
+                % included_datafile.dest_path
+            )
+
 
 class NuitkaPluginPyQt5QtPluginsPlugin(NuitkaPluginQtBindingsPluginBase):
     """This is for plugins of PyQt5.
@@ -1128,11 +1129,15 @@ class NuitkaPluginPyQt5QtPluginsPlugin(NuitkaPluginQtBindingsPluginBase):
 
     binding_name = "PyQt5"
 
-    def __init__(self, qt_plugins, no_qt_translations):
+    def __init__(self, include_qt_plugins, noinclude_qt_plugins, no_qt_translations):
         NuitkaPluginQtBindingsPluginBase.__init__(
-            self, qt_plugins=qt_plugins, no_qt_translations=no_qt_translations
+            self,
+            include_qt_plugins=include_qt_plugins,
+            noinclude_qt_plugins=noinclude_qt_plugins,
+            no_qt_translations=no_qt_translations,
         )
 
+        # TODO: make this into yaml instead, so we do not pollute this constructor.
         self.warning(
             """\
 For the obsolete PyQt5 the Nuitka support is incomplete. Threading, callbacks \
@@ -1209,7 +1214,7 @@ class NuitkaPluginPySide2Plugins(NuitkaPluginQtBindingsPluginBase):
 
     binding_name = "PySide2"
 
-    def __init__(self, qt_plugins, no_qt_translations):
+    def __init__(self, include_qt_plugins, noinclude_qt_plugins, no_qt_translations):
         if self._getNuitkaPatchLevel() < 1:
             self.warning(
                 """\
@@ -1225,7 +1230,10 @@ The standard PySide2 is not supported before CPython <3.6. For full support: htt
                 )
 
         NuitkaPluginQtBindingsPluginBase.__init__(
-            self, qt_plugins=qt_plugins, no_qt_translations=no_qt_translations
+            self,
+            include_qt_plugins=include_qt_plugins,
+            noinclude_qt_plugins=noinclude_qt_plugins,
+            no_qt_translations=no_qt_translations,
         )
 
     def onModuleEncounter(
@@ -1332,6 +1340,7 @@ def my_init_subclass(cls, *args):
 
 import PySide2.QtCore
 PySide2.QtCore.QAbstractItemModel.__init_subclass__ = my_init_subclass
+PySide2.QtCore.QAbstractTableModel.__init_subclass__ = my_init_subclass
 PySide2.QtCore.QObject.__init_subclass__ = my_init_subclass
 """
             yield (
@@ -1364,9 +1373,12 @@ class NuitkaPluginPySide6Plugins(NuitkaPluginQtBindingsPluginBase):
 
     binding_name = "PySide6"
 
-    def __init__(self, qt_plugins, no_qt_translations):
+    def __init__(self, include_qt_plugins, noinclude_qt_plugins, no_qt_translations):
         NuitkaPluginQtBindingsPluginBase.__init__(
-            self, qt_plugins=qt_plugins, no_qt_translations=no_qt_translations
+            self,
+            include_qt_plugins=include_qt_plugins,
+            noinclude_qt_plugins=noinclude_qt_plugins,
+            no_qt_translations=no_qt_translations,
         )
 
         if self._getBindingVersion() < (6, 5, 0):
@@ -1402,9 +1414,12 @@ class NuitkaPluginPyQt6Plugins(NuitkaPluginQtBindingsPluginBase):
 
     binding_name = "PyQt6"
 
-    def __init__(self, qt_plugins, no_qt_translations):
+    def __init__(self, include_qt_plugins, noinclude_qt_plugins, no_qt_translations):
         NuitkaPluginQtBindingsPluginBase.__init__(
-            self, qt_plugins=qt_plugins, no_qt_translations=no_qt_translations
+            self,
+            include_qt_plugins=include_qt_plugins,
+            noinclude_qt_plugins=noinclude_qt_plugins,
+            no_qt_translations=no_qt_translations,
         )
 
         self.info(
@@ -1412,8 +1427,6 @@ class NuitkaPluginPyQt6Plugins(NuitkaPluginQtBindingsPluginBase):
 Support for PyQt6 is not perfect, e.g. Qt threading does not work, so prefer PySide6 if you can."""
         )
 
-        return None
-
 
 class NuitkaPluginNoQt(NuitkaPluginBase):
     """This is a plugins for suppression of all Qt binding plugins."""
@@ -1433,8 +1446,8 @@ class NuitkaPluginNoQt(NuitkaPluginBase):
                 if top_package_name not in self.warned_about:
                     self.info(
                         """\
-Unwanted import of '%(unwanted)s' that is forbidden encountered, preventing
-its use. As a result an "ImportError" might be given at run time. Uninstall
+Unwanted import of '%(unwanted)s' that is forbidden encountered, preventing \
+its use. As a result an "ImportError" might be given at run time. Uninstall \
 it for full compatible behavior with the uncompiled code to debug it."""
                         % {
                             "unwanted": top_package_name,
@@ -1444,3 +1457,19 @@ it for full compatible behavior with the uncompiled code to debug it."""
                     self.warned_about.add(top_package_name)
 
                 return (False, "Not included due to all Qt bindings disallowed.")
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
