@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ This tool is generating node variants from Jinja templates.
 
 """
@@ -26,15 +12,18 @@ nuitka.Options.is_full_compat = False
 
 # isort:start
 
+import textwrap
 from collections import namedtuple
 
 import nuitka.code_generation.BinaryOperationHelperDefinitions
 import nuitka.code_generation.CodeGeneration
 import nuitka.code_generation.ComparisonCodes
 import nuitka.code_generation.Namify
+import nuitka.nodes.NetworkxNodes
 import nuitka.nodes.PackageMetadataNodes
 import nuitka.nodes.PackageResourceNodes
 import nuitka.nodes.SideEffectNodes
+import nuitka.nodes.TensorflowNodes
 import nuitka.specs.BuiltinBytesOperationSpecs
 import nuitka.specs.BuiltinDictOperationSpecs
 import nuitka.specs.BuiltinListOperationSpecs
@@ -262,20 +251,35 @@ lambda source_ref: wrapExpressionWithNodeSideEffects(
 
 
 def emitGenerationWarning(emit, doc_string, template_name):
-    attribute_code_names = set(attribute_information.keys())
-    attribute_code_names = set(
+    generate_names = set()
+
+    generate_names.update(attribute_information.keys())
+    generate_names.update(
         attribute_name.replace("_", "") for attribute_name in attribute_information
     )
 
-    attribute_arg_names = set(sum(attribute_shape_args.values(), ()))
+    generate_names.update(sum(attribute_shape_args.values(), ()))
+
+    for spec_descriptions in getSpecVersions(nuitka.specs.HardImportSpecs):
+        spec = spec_descriptions[0][2]
+        generate_names.update(spec.getArgumentNames())
+
+    ignores = textwrap.fill(
+        " ".join(sorted(generate_names)),
+        width=90,
+        initial_indent="spell-checker: ignore ",
+        subsequent_indent="spell-checker: ignore ",
+        break_on_hyphens=False,
+        break_long_words=False,
+        expand_tabs=False,
+        replace_whitespace=False,
+    )
 
     emit(
         """
 # We are not avoiding these in generated code at all
-# pylint: disable=I0021,too-many-lines
-# pylint: disable=I0021,line-too-long
-# pylint: disable=I0021,too-many-instance-attributes
-# pylint: disable=I0021,too-many-return-statements
+# pylint: disable=I0021,line-too-long,too-many-instance-attributes,too-many-lines
+# pylint: disable=I0021,too-many-arguments,too-many-return-statements,too-many-statements
 """
     )
 
@@ -285,17 +289,11 @@ def emitGenerationWarning(emit, doc_string, template_name):
 
 WARNING, this code is GENERATED. Modify the template %s instead!
 
-spell-checker: ignore %s
-spell-checker: ignore %s
+%s
 """
 
 '''
-        % (
-            doc_string,
-            template_name,
-            " ".join(sorted(attribute_code_names)),
-            " ".join(sorted(attribute_arg_names)),
-        )
+        % (doc_string, template_name, ignores)
     )
 
 
@@ -376,7 +374,6 @@ def makeAttributeNodes():
         emit("attribute_typed_classes = set()")
 
         for attribute_name, shape_names in sorted(attribute_information.items()):
-
             code = template.render(
                 attribute_name=attribute_name,
                 python3_operation_name=_getPython3OperationName(attribute_name),
@@ -450,6 +447,9 @@ def adaptModuleName(value):
     if value == "importlib_metadata":
         return "importlib_metadata_backport"
 
+    if value == "importlib_resources":
+        return "importlib_resources_backport"
+
     return value
 
 
@@ -462,17 +462,28 @@ def makeCodeCased(value):
 
 
 def getCallModuleName(module_name, function_name):
+    # return driven, pylint: disable=too-many-return-statements
+
     if module_name in ("pkg_resources", "importlib.metadata", "importlib_metadata"):
         if function_name in ("resource_stream", "resource_string"):
             return "PackageResourceNodes"
 
         return "PackageMetadataNodes"
-    if module_name in ("pkgutil", "importlib.resources"):
+    if module_name in ("pkgutil", "importlib.resources", "importlib_resources"):
         return "PackageResourceNodes"
     if module_name in ("os", "sys", "os.path"):
         return "OsSysNodes"
     if module_name == "ctypes":
         return "CtypesNodes"
+    if module_name == "builtins":
+        if function_name == "open":
+            return "BuiltinOpenNodes"
+
+    if module_name == "tensorflow":
+        return "TensorflowNodes"
+
+    if module_name.startswith("networkx"):
+        return "NetworkxNodes"
 
     assert False, (module_name, function_name)
 
@@ -625,14 +636,14 @@ def _parseNamedChildrenSpec(named_children):
                     named_children_checkers[named_child] = "convertNoneConstantToNone"
                 elif named_child_property == "auto_none_empty_str":
                     named_children_types[named_child] = "optional"
-                    named_children_checkers[
-                        named_child
-                    ] = "convertEmptyStrConstantToNone"
+                    named_children_checkers[named_child] = (
+                        "convertEmptyStrConstantToNone"
+                    )
                 elif named_child_property == "statements_or_none":
                     named_children_types[named_child] = "optional"
-                    named_children_checkers[
-                        named_child
-                    ] = "checkStatementsSequenceOrNone"
+                    named_children_checkers[named_child] = (
+                        "checkStatementsSequenceOrNone"
+                    )
                 elif named_child_property == "statements":
                     named_children_checkers[named_child] = "checkStatementsSequence"
                 elif named_child_property == "optional":
@@ -676,8 +687,9 @@ def _addFromNode(node_class):
     ) = _parseNamedChildrenSpec(named_children)
 
     mixin_name = makeMixinName(
-        node_class.isExpression(),
-        node_class.isStatement() or node_class.isStatementsSequence(),
+        # TODO: Subject to dying, we now make this up on the fly.
+        node_class.kind.startswith("EXPRESSION"),
+        node_class.kind.startswith("STATEMENT"),
         tuple(new_named_children),
         named_children_types,
         named_children_checkers,
@@ -697,8 +709,9 @@ def _addFromNode(node_class):
         print("Not done", node_class.__name__, named_children, mixin_name)
 
     addChildrenMixin(
-        node_class.isExpression(),
-        node_class.isStatement() or node_class.isStatementsSequence(),
+        # TODO: Subject to dying, we now make this up on the fly.
+        node_class.kind.startswith("EXPRESSION"),
+        node_class.kind.startswith("STATEMENT"),
         node_class.__name__,
         tuple(new_named_children),
         named_children_types,
@@ -719,12 +732,16 @@ def addFromNodes():
         _addFromNode(node_class)
 
     # Fake factories:
-    node_factory_translations[
-        "ExpressionImportlibMetadataMetadataCall"
-    ] = "makeExpressionImportlibMetadataMetadataCall"
-    node_factory_translations[
-        "ExpressionImportlibMetadataBackportMetadataCall"
-    ] = "makeExpressionImportlibMetadataBackportMetadataCall"
+    node_factory_translations["ExpressionImportlibMetadataMetadataCall"] = (
+        "makeExpressionImportlibMetadataMetadataCall"
+    )
+    node_factory_translations["ExpressionImportlibMetadataBackportMetadataCall"] = (
+        "makeExpressionImportlibMetadataBackportMetadataCall"
+    )
+    node_factory_translations["ExpressionBuiltinsOpenCall"] = (
+        "makeExpressionBuiltinsOpenCall"
+    )
+    node_factory_translations["ExpressionSysExitCall"] = "makeExpressionSysExitCall"
 
 
 addFromNodes()
@@ -841,6 +858,8 @@ from nuitka.nodes.Checkers import (
 
             is_compute_final = pop("final")
 
+            is_compute_final_children = pop("final_children")
+
             is_compute_no_raise = pop("no_raise")
             is_compute_raise = pop("raise")
             is_compute_raise_operation = pop("raise_operation")
@@ -885,6 +904,7 @@ from nuitka.nodes.Checkers import (
                 ),
                 intended_for=intended_for,
                 is_compute_final=is_compute_final,
+                is_compute_final_children=is_compute_final_children,
                 raise_mode=raise_mode,
                 is_compute_statement=is_compute_statement,
                 awaited_constant_attributes=awaited_constant_attributes,
@@ -912,7 +932,7 @@ def getSpecVersions(spec_module):
     result = {}
 
     for spec_name, spec in getSpecs(spec_module):
-        for (version, str_version) in (
+        for version, str_version in (
             (0x370, "37"),
             (0x380, "38"),
             (0x390, "39"),
@@ -1024,9 +1044,16 @@ hard_import_node_classes = {}
                         assert optional_name not in named_children_types
                         named_children_types[optional_name] = "optional"
 
+                if spec.getStarListArgumentName():
+                    named_children_types[spec.getStarListArgumentName()] = "tuple"
+
                 if spec.getStarDictArgumentName():
                     named_children_types[spec.getStarDictArgumentName()] = "tuple"
 
+                for kw_only_name in spec.getKwOnlyParameterNames():
+                    assert kw_only_name not in named_children_types
+                    named_children_types[kw_only_name] = "optional"
+
                 if parameter_names:
                     mixin_name = addChildrenMixin(
                         True,
@@ -1061,6 +1088,16 @@ hard_import_node_classes = {}
                     function_name_title=function_name_title,
                     function_name_code=makeCodeCased(function_name),
                     module_name=module_name,
+                    is_stdlib_module=module_name
+                    in (
+                        "builtins",
+                        "os",
+                        "os.path",
+                        "pkgutil",
+                        "ctypes",
+                        "importlib.metadata",
+                        "importlib.resources",
+                    ),
                     module_name_code=makeCodeCased(adaptModuleName(module_name)),
                     module_name_title=module_name_title,
                     call_node_module_name=getCallModuleName(module_name, function_name),
@@ -1075,3 +1112,19 @@ def main():
     makeAttributeNodes()
     makeBuiltinOperationNodes()
     makeChildrenHavingMixinNodes()
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
