@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Tools to compare outputs of compiled and not compiled programs.
 
 There is a couple of replacements to be done for compiled programs to
@@ -70,7 +56,7 @@ def import_re_callback(match):
 
 tempfile_re = re.compile(r"/tmp/tmp[a-z0-9_]*")
 
-logging_info_re = re.compile(r"^Nuitka.*?:INFO")
+logging_info_re = re.compile(r"^Nuitka(-\w+)?:([-\w]+:)? ")
 logging_warning_re = re.compile(r"^Nuitka.*?:WARNING")
 
 # Python3.11 style traceback carets are not done by Nuitka (yet?)
@@ -78,6 +64,8 @@ syntax_error_caret_re = re.compile(r"^\s*~*\^*~*$")
 
 timing_re = re.compile(r"in [0-9]+.[0-9][0-9](s| seconds)")
 
+did_you_mean_re = re.compile(r"\. Did you mean: '.*?'\?")
+
 
 def makeDiffable(output, ignore_warnings, syntax_errors):
     # Of course many cases to deal with,
@@ -92,7 +80,6 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
 
     lines = output.split(b"\n")
     if syntax_errors:
-
         for line in lines:
             if line.startswith(b"SyntaxError:"):
                 lines = [line]
@@ -137,6 +124,8 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
             continue
         if line.startswith("Nuitka:WARNING:     Complex topic"):
             continue
+        if line.startswith("Nuitka:WARNING:") and "matching checksum" in line:
+            continue
 
         if syntax_error_caret_re.match(line):
             continue
@@ -145,7 +134,30 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
         line = thread_re.sub(r"Thread 0xXXXXXXXX", line)
         line = compiled_types_re.sub(r"\1", line)
         line = global_name_error_re.sub(r"\1\2\3", line)
+
         line = module_repr_re.sub(r"\1xxxxx\2", line)
+
+        # Frozen modules of 3.11, _imp._frozen_module_names
+        for module_name in (
+            "zipimport",
+            "abc",
+            "codecs",
+            "io",
+            "_collections_abc",
+            "_sitebuiltins",
+            "genericpath",
+            "ntpath",
+            "posixpath",
+            "os.path",
+            "os",
+            "site",
+            "stat",
+        ):
+            line = line.replace(
+                "<module '%s' (frozen)>" % module_name,
+                "<module '%s' from 'xxxxx'>" % module_name,
+            )
+
         line = non_ascii_error_rt.sub(r"\1 xxxx", line)
         line = timing_re.sub(r"in x.xx seconds", line)
 
@@ -167,6 +179,8 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
 
         line = tempfile_re.sub(r"/tmp/tmpxxxxxxx", line)
 
+        line = did_you_mean_re.sub("", line)
+
         # This is a bug potentially, occurs only for CPython when re-directed,
         # we are going to ignore the issue as Nuitka is fine.
         if (
@@ -220,6 +234,10 @@ exceeded while calling a Python object' in \
         if "clcache: persistent json file" in line or "clcache: manifest file" in line:
             continue
 
+        # Some tests do malloc too large things on purpose
+        if "WARNING: AddressSanitizer failed to allocate" in line:
+            continue
+
         # Ignore manual error message of CPython 3.11 that is different from the generic one for super
         line = line.replace(
             "super() argument 1 must be a type, not NoneType",
@@ -229,6 +247,11 @@ exceeded while calling a Python object' in \
             "super() argument 1 must be a type", "super() argument 1 must be type"
         )
 
+        # Ignore logger message on macOS that will have timestamps and program name
+        # that differ without forcing logger config to avoid it.
+        if "XType: Using static font registry" in line:
+            continue
+
         result.append(line)
 
     return result
@@ -260,3 +283,19 @@ def compareOutput(
         return 1
     else:
         return 0
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
