@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Build the internal node tree from source code.
 
 Does all the Python parsing and puts it into a tree structure for use in later
@@ -48,6 +34,7 @@ special attribute lookups for "__enter__" and "__exit__", calls of them,
 catching and passing in exceptions raised.
 
 """
+
 import marshal
 import os
 
@@ -62,9 +49,13 @@ from nuitka.BytecodeCaching import (
     getCachedImportedModuleUsageAttempts,
     hasCachedImportedModuleUsageAttempts,
 )
+from nuitka.Bytecodes import loadCodeObjectData
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.Errors import CodeTooComplexCode
-from nuitka.freezer.Standalone import detectEarlyImports
+from nuitka.freezer.ImportDetection import (
+    detectEarlyImports,
+    detectStdlibAutoInclusionModules,
+)
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.importing.PreloadedPackages import getPthImportedPackages
@@ -88,13 +79,15 @@ from nuitka.nodes.ConstantRefNodes import (
     ExpressionConstantNoneRef,
     makeConstantRefNode,
 )
-from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import (
     StatementRaiseException,
     StatementReraiseException,
 )
 from nuitka.nodes.FutureSpecs import FutureSpec
-from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
+from nuitka.nodes.GeneratorNodes import (
+    StatementGeneratorReturn,
+    StatementGeneratorReturnNone,
+)
 from nuitka.nodes.ImportNodes import (
     isHardModuleWithoutSideEffect,
     makeExpressionImportModuleFixed,
@@ -124,7 +117,6 @@ from nuitka.nodes.VariableNameNodes import (
     ExpressionVariableNameRef,
     StatementAssignmentVariableName,
 )
-from nuitka.nodes.YieldNodes import ExpressionYieldFromWaitable
 from nuitka.optimizations.BytecodeDemotion import demoteSourceCodeToBytecode
 from nuitka.Options import shallWarnUnusualCode
 from nuitka.pgo.PGO import decideCompilationFromPGO
@@ -148,6 +140,7 @@ from .ReformulationAssignmentStatements import (
     buildDeleteNode,
     buildInplaceAssignNode,
     buildNamedExprNode,
+    buildTypeAliasNode,
 )
 from .ReformulationBooleanExpressions import buildBoolOpNode
 from .ReformulationCallExpressions import buildCallNode
@@ -180,7 +173,6 @@ from .ReformulationImportStatements import (
 from .ReformulationLambdaExpressions import buildLambdaNode
 from .ReformulationMatchStatements import buildMatchNode
 from .ReformulationNamespacePackages import (
-    createImporterCacheAssignment,
     createNamespacePackage,
     createPathAssignment,
 )
@@ -191,11 +183,18 @@ from .ReformulationSequenceCreation import (
     buildTupleCreationNode,
 )
 from .ReformulationSubscriptExpressions import buildSubscriptNode
-from .ReformulationTryExceptStatements import buildTryExceptionNode
+from .ReformulationTryExceptStatements import (
+    buildTryExceptionNode,
+    buildTryStarExceptionNode,
+)
 from .ReformulationTryFinallyStatements import buildTryFinallyNode
 from .ReformulationWhileLoopStatements import buildWhileLoopNode
 from .ReformulationWithStatements import buildAsyncWithNode, buildWithNode
-from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
+from .ReformulationYieldExpressions import (
+    buildAwaitNode,
+    buildYieldFromNode,
+    buildYieldNode,
+)
 from .SourceHandling import (
     checkPythonVersionFromCode,
     getSourceCodeDiff,
@@ -254,7 +253,7 @@ def buildNamedConstantNode(node, source_ref):
 def buildConditionNode(provider, node, source_ref):
     # Conditional statements may have one or two branches. We will never see an
     # "elif", because that's already dealt with by module "ast", which turns it
-    # into nested conditional statements.
+    # into nested conditional statements. spell-checker: ignore orelse
 
     return makeStatementConditional(
         condition=buildNode(provider, node.test, source_ref),
@@ -292,7 +291,7 @@ def buildTryNode(provider, node, source_ref):
     if not node.handlers:
         return buildTryFinallyNode2(provider, node, source_ref)
 
-    if not node.finalbody:
+    if not node.finalbody:  # spell-checker: ignore finalbody
         return buildTryExceptionNode(
             provider=provider, node=node, source_ref=source_ref
         )
@@ -316,10 +315,42 @@ def buildTryNode(provider, node, source_ref):
     )
 
 
+def buildTryStarNode(provider, node, source_ref):
+    # Note: This variant is used for Python3.11 or higher only, where an exception
+    # group is caught. Mixing groups and non-group catches is not allowed.
+
+    # Without handlers, this would not be used, but instead "Try" would be used,
+    # but assert against it.
+    assert node.handlers
+
+    if not node.finalbody:  # spell-checker: ignore finalbody
+        return buildTryStarExceptionNode(
+            provider=provider, node=node, source_ref=source_ref
+        )
+
+    return buildTryFinallyNode(
+        provider=provider,
+        build_tried=lambda: makeStatementsSequence(
+            statements=mergeStatements(
+                (
+                    buildTryStarExceptionNode(
+                        provider=provider, node=node, source_ref=source_ref
+                    ),
+                ),
+                allow_none=True,
+            ),
+            allow_none=True,
+            source_ref=source_ref,
+        ),
+        node=node,
+        source_ref=source_ref,
+    )
+
+
 def buildRaiseNode(provider, node, source_ref):
     # Raise statements. Under Python2 they may have type, value and traceback
     # attached, for Python3, you can only give type (actually value) and cause.
-
+    # spell-checker: ignore tback
     if python_version < 0x300:
         exception_type = buildNode(provider, node.type, source_ref, allow_none=True)
         exception_value = buildNode(provider, node.inst, source_ref, allow_none=True)
@@ -367,7 +398,6 @@ def buildRaiseNode(provider, node, source_ref):
 
 
 def handleGlobalDeclarationNode(provider, node, source_ref):
-
     # On the module level, there is nothing to do.
     if provider.isCompiledPythonModule():
         if shallWarnUnusualCode():
@@ -401,7 +431,7 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
     for variable_name in node.names:
         closure_variable = None
 
-        # Re-use already taken global variables, in order to avoid creating yet
+        # Reuse already taken global variables, in order to avoid creating yet
         # another instance, esp. as the indications could then potentially not
         # be shared.
         if provider.hasTakenVariable(variable_name):
@@ -555,10 +585,12 @@ def buildReturnNode(provider, node, source_ref):
         provider.isExpressionGeneratorObjectBody()
         or provider.isExpressionAsyncgenObjectBody()
     ):
-        if expression is None:
-            expression = ExpressionConstantNoneRef(source_ref=source_ref)
-
-        return StatementGeneratorReturn(expression=expression, source_ref=source_ref)
+        if expression is None or expression.isExpressionConstantNoneRef():
+            return StatementGeneratorReturnNone(source_ref=source_ref)
+        else:
+            return StatementGeneratorReturn(
+                expression=expression, source_ref=source_ref
+            )
     else:
         return makeStatementReturn(expression=expression, source_ref=source_ref)
 
@@ -624,16 +656,6 @@ def buildConditionalExpressionNode(provider, node, source_ref):
     )
 
 
-def buildAwaitNode(provider, node, source_ref):
-    return ExpressionYieldFromWaitable(
-        expression=ExpressionAsyncWait(
-            expression=buildNode(provider, node.value, source_ref),
-            source_ref=source_ref,
-        ),
-        source_ref=source_ref,
-    )
-
-
 def buildFormattedValueNode(provider, node, source_ref):
     value = buildNode(provider, node.value, source_ref)
 
@@ -710,6 +732,8 @@ setBuildingDispatchers(
         "TryFinally": buildTryFinallyNode2,
         "Try": buildTryNode,
         "Raise": buildRaiseNode,
+        # Python3.11 exception group catching
+        "TryStar": buildTryStarNode,
         "Import": buildImportModulesNode,
         "ImportFrom": buildImportFromNode,
         "Assert": buildAssertNode,
@@ -740,6 +764,7 @@ setBuildingDispatchers(
         "NamedExpr": buildNamedExprNode,
         "Slice": buildSliceNode,
         "Match": buildMatchNode,
+        "TypeAlias": buildTypeAliasNode,
     },
     path_args2={
         "Constant": buildNamedConstantNode,  # Python3.8
@@ -793,7 +818,10 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
             statements.append(
                 StatementExpressionOnly(
                     expression=makeExpressionImportModuleFixed(
-                        module_name="site", source_ref=source_ref
+                        using_module_name=provider.getParentModule().getFullName(),
+                        module_name="site",
+                        value_name="site",
+                        source_ref=source_ref,
                     ),
                     source_ref=source_ref,
                 )
@@ -806,7 +834,10 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
                 statements.append(
                     StatementExpressionOnly(
                         expression=makeExpressionImportModuleFixed(
-                            module_name=path_imported_name, source_ref=source_ref
+                            using_module_name=provider.getParentModule().getFullName(),
+                            module_name=path_imported_name,
+                            value_name=path_imported_name.getTopLevelPackageName(),
+                            source_ref=source_ref,
                         ),
                         source_ref=source_ref,
                     )
@@ -838,9 +869,6 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
         if provider.isCompiledPythonPackage():
             # This assigns "__path__" value.
             statements.append(createPathAssignment(provider, internal_source_ref))
-            statements.append(
-                createImporterCacheAssignment(provider, internal_source_ref)
-            )
 
         if python_version >= 0x340 and not is_main:
             statements += (
@@ -953,13 +981,19 @@ def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
         assert False
 
 
-def decideCompilationMode(is_top, module_name, for_pgo):
+def decideCompilationMode(is_top, module_name, module_filename, for_pgo):
     """Decide the compilation mode for a module.
 
     module_name - The module to decide compilation mode for.
     for_pgo - consider PGO information or not
     """
 
+    is_stdlib = module_filename is not None and isStandardLibraryPath(module_filename)
+
+    # Technically required modules must be bytecode
+    if is_stdlib and module_name in detectEarlyImports():
+        return "bytecode"
+
     result = Plugins.decideCompilation(module_name)
 
     # Cannot change mode of __main__ to bytecode, that is not going
@@ -976,13 +1010,8 @@ required to compiled."""
 
     # Include all of standard library as bytecode, for now. We need to identify
     # which ones really need that.
-    if not is_top:
-        module_filename = Importing.locateModule(
-            module_name=module_name, parent_package=None, level=0
-        )[1]
-
-        if module_filename is not None and isStandardLibraryPath(module_filename):
-            result = "bytecode"
+    if not is_top and is_stdlib:
+        result = "bytecode"
 
     # Plugins need to win over PGO, as they might know it better
     if result is None and not for_pgo:
@@ -990,22 +1019,27 @@ required to compiled."""
 
     # Default if neither plugins nor PGO have expressed an opinion
     if result is None:
-        result = "compiled"
+        if is_stdlib and module_name in detectStdlibAutoInclusionModules():
+            result = "bytecode"
+        else:
+            result = "compiled"
 
     return result
 
 
-def _loadUncompiledModuleFromCache(module_name, is_package, source_code, source_ref):
+def _loadUncompiledModuleFromCache(
+    module_name, reason, is_package, source_code, source_ref
+):
     result = makeUncompiledPythonModule(
         module_name=module_name,
+        reason=reason,
         filename=source_ref.getFilename(),
         bytecode=demoteSourceCodeToBytecode(
             module_name=module_name,
             source_code=source_code,
             filename=source_ref.getFilename(),
         ),
-        user_provided=False,
-        technical=False,
+        technical=module_name in detectEarlyImports(),
         is_package=is_package,
     )
 
@@ -1024,25 +1058,37 @@ def _loadUncompiledModuleFromCache(module_name, is_package, source_code, source_
 
 def _createModule(
     module_name,
+    module_filename,
+    module_kind,
+    reason,
     source_code,
     source_ref,
-    is_extension,
     is_namespace,
     is_package,
     is_top,
     is_main,
     main_added,
 ):
-    if is_extension:
+    is_stdlib = module_filename is not None and isStandardLibraryPath(module_filename)
+
+    if module_kind == "extension":
         result = PythonExtensionModule(
-            module_name=module_name, technical=False, source_ref=source_ref
+            module_name=module_name,
+            reason=reason,
+            technical=is_stdlib and module_name in detectEarlyImports(),
+            source_ref=source_ref,
         )
     elif is_main:
+        assert reason == "main", reason
+
         result = PythonMainModule(
             main_added=main_added,
             module_name=module_name,
             mode=decideCompilationMode(
-                is_top=is_top, module_name=module_name, for_pgo=False
+                is_top=is_top,
+                module_name=module_name,
+                module_filename=module_filename,
+                for_pgo=False,
             ),
             future_spec=None,
             source_ref=source_ref,
@@ -1050,10 +1096,18 @@ def _createModule(
 
         checkPythonVersionFromCode(source_code)
     elif is_namespace:
-        result = createNamespacePackage(module_name, is_top, source_ref)
+        result = createNamespacePackage(
+            module_name=module_name,
+            reason=reason,
+            is_top=is_top,
+            source_ref=source_ref,
+        )
     else:
         mode = decideCompilationMode(
-            is_top=is_top, module_name=module_name, for_pgo=False
+            is_top=is_top,
+            module_name=module_name,
+            module_filename=module_filename,
+            for_pgo=False,
         )
 
         if (
@@ -1066,6 +1120,7 @@ def _createModule(
         ):
             result = _loadUncompiledModuleFromCache(
                 module_name=module_name,
+                reason=reason,
                 is_package=is_package,
                 source_code=source_code,
                 source_ref=source_ref,
@@ -1077,6 +1132,7 @@ def _createModule(
             if is_package:
                 result = CompiledPythonPackage(
                     module_name=module_name,
+                    reason=reason,
                     is_top=is_top,
                     mode=mode,
                     future_spec=None,
@@ -1085,6 +1141,7 @@ def _createModule(
             else:
                 result = CompiledPythonModule(
                     module_name=module_name,
+                    reason=reason,
                     is_top=is_top,
                     mode=mode,
                     future_spec=None,
@@ -1119,32 +1176,42 @@ def createModuleTree(module, source_ref, ast_tree, is_main):
         )
 
 
-def buildMainModuleTree(filename, is_main, source_code):
+def buildMainModuleTree(filename, source_code):
     # Detect to be frozen modules if any, so we can consider to not follow
     # to them.
 
-    if is_main:
+    if Options.shallMakeModule():
+        module_name = Importing.getModuleNameAndKindFromFilename(filename)[0]
+
+        if module_name is None:
+            general.sysexit(
+                "Error, filename '%s' suffix does not appear to be Python module code."
+                % filename
+            )
+    else:
         # TODO: Doesn't work for deeply nested packages at all.
         if Options.hasPythonFlagPackageMode():
             module_name = ModuleName(os.path.basename(filename) + ".__main__")
         else:
             module_name = ModuleName("__main__")
-    else:
-        module_name = Importing.getModuleNameAndKindFromFilename(filename)[0]
 
-    module, _added = buildModule(
+    module = buildModule(
         module_name=module_name,
+        reason="main",
         module_filename=filename,
         source_code=source_code,
         is_top=True,
-        is_main=is_main,
-        is_extension=False,
+        is_main=not Options.shallMakeModule(),
+        module_kind="py",
         is_fake=source_code is not None,
         hide_syntax_error=False,
     )
 
-    if is_main and Options.isStandaloneMode():
-        module.setEarlyModules(detectEarlyImports())
+    if Options.isStandaloneMode():
+        module.setStandardLibraryModules(
+            early_module_names=detectEarlyImports(),
+            stdlib_modules_names=detectStdlibAutoInclusionModules(),
+        )
 
     # Main modules do not get added to the import cache, but plugins get to see it.
     if module.isMainModule():
@@ -1155,7 +1222,7 @@ def buildMainModuleTree(filename, is_main, source_code):
     return module
 
 
-def _makeModuleBodyFromSyntaxError(exc, module_name, module_filename):
+def _makeModuleBodyFromSyntaxError(exc, module_name, reason, module_filename):
     if module_filename not in Importing.warned_about:
         Importing.warned_about.add(module_filename)
 
@@ -1169,6 +1236,7 @@ Cannot follow import to module '%s' because of '%s'."""
 
     module = CompiledPythonModule(
         module_name=module_name,
+        reason=reason,
         is_top=False,
         mode="compiled",
         future_spec=FutureSpec(),
@@ -1191,39 +1259,40 @@ Cannot follow import to module '%s' because of '%s'."""
     return module
 
 
-def _makeModuleBodyTooComplex(module_name, module_filename, source_code, is_package):
+def _makeModuleBodyTooComplex(
+    module_name, reason, module_filename, source_code, is_package
+):
     if module_filename not in Importing.warned_about:
         Importing.warned_about.add(module_filename)
 
-        recursion_logger.info(
-            """\
+        # Known harmless case, not causing issues, lets not warn about it.
+        if module_name != "sympy.polys.numberfields.resolvent_lookup":
+            recursion_logger.info(
+                """\
 Cannot compile module '%s' because its code is too complex, included as bytecode."""
-            % module_name
-        )
+                % module_name
+            )
 
-    module = makeUncompiledPythonModule(
+    return makeUncompiledPythonModule(
         module_name=module_name,
+        reason=reason,
         filename=module_filename,
         bytecode=marshal.dumps(
             compile(source_code, module_filename, "exec", dont_inherit=True)
         ),
         is_package=is_package,
-        user_provided=True,
-        technical=False,
+        technical=module_name in detectEarlyImports(),
     )
 
-    ModuleRegistry.addUncompiledModule(module)
-
-    return module
-
 
 def buildModule(
     module_name,
+    module_kind,
     module_filename,
+    reason,
     source_code,
     is_top,
     is_main,
-    is_extension,
     is_fake,
     hide_syntax_error,
 ):
@@ -1253,9 +1322,20 @@ def buildModule(
                 "Python flag -m (package_mode) only works on packages with '__main__.py'."
             )
 
+    # Handle bytecode module case immediately.
+    if module_kind == "pyc":
+        return makeUncompiledPythonModule(
+            module_name=module_name,
+            reason=reason,
+            filename=module_filename,
+            bytecode=loadCodeObjectData(module_filename),
+            is_package=is_package,
+            technical=module_name in detectEarlyImports(),
+        )
+
     # Read source code if necessary. Might give a SyntaxError due to not being proper
     # encoded source.
-    if source_filename is not None and not is_namespace and not is_extension:
+    if source_filename is not None and not is_namespace and module_kind == "py":
         # For fake modules, source is provided directly.
         original_source_code = None
         contributing_plugins = ()
@@ -1278,10 +1358,12 @@ def buildModule(
                 if not hide_syntax_error:
                     raise
 
-                module = _makeModuleBodyFromSyntaxError(
-                    exc=e, module_name=module_name, module_filename=module_filename
+                return _makeModuleBodyFromSyntaxError(
+                    exc=e,
+                    module_name=module_name,
+                    reason=reason,
+                    module_filename=module_filename,
                 )
-                return module, True
 
         try:
             ast_tree = parseSourceCodeToAst(
@@ -1310,7 +1392,7 @@ def buildModule(
                     source_diff = getSourceCodeDiff(original_source_code, source_code)
 
                     for line in source_diff:
-                        plugins_logger.warning(line)
+                        plugins_logger.warning(line, keep_format=True)
 
                     if len(contributing_plugins) == 1:
                         next(iter(contributing_plugins)).sysexit(
@@ -1323,33 +1405,37 @@ def buildModule(
                             % (",".join(contributing_plugins), module_name, e)
                         )
 
-            module = _makeModuleBodyFromSyntaxError(
-                exc=e, module_name=module_name, module_filename=module_filename
+            return _makeModuleBodyFromSyntaxError(
+                exc=e,
+                module_name=module_name,
+                reason=reason,
+                module_filename=module_filename,
             )
-            return module, True
         except CodeTooComplexCode:
             # Do not hide CodeTooComplexCode in main module.
             if is_main:
                 raise
 
-            module = _makeModuleBodyTooComplex(
+            return _makeModuleBodyTooComplex(
                 module_name=module_name,
+                reason=reason,
                 module_filename=module_filename,
                 source_code=source_code,
                 is_package=is_package,
             )
-            return module, False
     else:
         ast_tree = None
         source_code = None
 
     module = _createModule(
         module_name=module_name,
+        module_filename=None if is_fake else module_filename,
+        module_kind=module_kind,
+        reason=reason,
         source_code=source_code,
         source_ref=source_ref,
         is_top=is_top,
         is_main=is_main,
-        is_extension=is_extension,
         is_namespace=is_namespace,
         is_package=is_package,
         main_added=main_added,
@@ -1373,13 +1459,28 @@ def buildModule(
             if is_main or is_top:
                 raise
 
-            module = _makeModuleBodyTooComplex(
+            return _makeModuleBodyTooComplex(
                 module_name=module_name,
+                reason=reason,
                 module_filename=module_filename,
                 source_code=source_code,
                 is_package=is_package,
             )
 
-            return module, False
+    return module
 
-    return module, True
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
