@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Helper functions for parsing the AST nodes and building the Nuitka node tree.
 
 """
@@ -142,7 +128,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body"):
                     pass
-                elif name in ("bases", "decorator_list", "keywords"):
+                elif name in ("bases", "decorator_list", "keywords", "type_params"):
                     for child in field:
                         _check(child)
                 elif name == "starargs":
@@ -159,7 +145,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body"):
                     pass
-                elif name in ("bases", "decorator_list"):
+                elif name in ("bases", "decorator_list", "type_params"):
                     for child in field:
                         _check(child)
                 elif name == "args":
@@ -211,7 +197,10 @@ def detectFunctionBodyKind(nodes, start_value=None):
                 if name in ("name", "body", "comparators"):
                     pass
                 elif name == "generators":
-                    _check(field[0].iter)
+                    if python_version < 0x3B0:
+                        _check(field[0].iter)
+                    else:
+                        _check(field[0])
                 elif name in ("body", "elt"):
                     _check(field)
                 else:
@@ -396,7 +385,6 @@ def buildAnnotationNode(provider, node, source_ref):
         python_version >= 0x370
         and provider.getParentModule().getFutureSpec().isFutureAnnotations()
     ):
-
         # Using global value for cache, to avoid creating it over and over,
         # avoiding the pylint: disable=global-statement
         global _host_node
@@ -692,3 +680,19 @@ def popBuildContext():
 
 def getBuildContext():
     return build_contexts[-1]
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
