@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ C string encoding
 
 This contains the code to create string literals for C to represent the given
@@ -24,7 +10,7 @@ values.
 import codecs
 import re
 
-from nuitka.__past__ import unicode
+from nuitka.__past__ import to_byte, unicode
 
 
 def _identifierEncode(c):
@@ -52,7 +38,7 @@ def _encodePythonStringToC(value):
             cv = c
 
         if c in b'\\\t\r\n"?':
-            result += r"\%o" % cv
+            result += r"\%03o" % cv
 
             octal = True
         elif 32 <= cv <= 127:
@@ -81,7 +67,12 @@ def encodePythonUnicodeToC(value):
     for c in value:
         cv = ord(c)
 
-        result += r"\%o" % cv
+        if c == "\\":
+            result += "\\\\"
+        elif 34 < cv < 128:
+            result += c
+        else:
+            result += r"\x%04x" % cv
 
     return 'L"%s"' % result
 
@@ -105,6 +96,37 @@ def encodePythonStringToC(value):
     return result
 
 
+def decodeCStringToPython(value):
+    assert value.startswith('"')
+    value = value[1:-1]
+
+    result = b""
+
+    while value:
+        if value[0] == "\\":
+            value = value[1:]
+            if value[0] == "\\":
+                result += b"\\"
+                continue
+
+            if value[0].isdigit():
+                assert value[1].isdigit(), value
+                assert value[2].isdigit(), value
+
+                octal_number = int(value[2]) + int(value[1]) * 8 + int(value[0]) * 64
+                value = value[3:]
+
+                result += to_byte(octal_number)
+                continue
+
+            assert False, value[0]
+
+        result += value[0].encode("ascii")
+        value = value[1:]
+
+    return result
+
+
 def encodePythonIdentifierToC(value):
     """Encode an identifier from a given Python string."""
 
@@ -120,4 +142,20 @@ def encodePythonIdentifierToC(value):
         else:
             return "$$%d$" % ord(c)
 
-    return "".join(re.sub("[^a-zA-Z0-9_]", r, c) for c in value)
+    return re.sub("[^a-zA-Z0-9_]", r, value)
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
