@@ -97,6 +97,9 @@ def areSamePaths(path1, path2):
         even short paths, it then becomes a mere string compare after that.
     """
 
+    if path1 == path2:
+        return True
+
     path1 = os.path.abspath(os.path.normpath(path1))
     path2 = os.path.abspath(os.path.normpath(path2))
 
@@ -186,7 +189,11 @@ def makePath(path):
 
     with withFileLock("creating directory %s" % path):
         if not os.path.isdir(path):
-            os.makedirs(path)
+            try:
+                os.makedirs(path)
+            except OSError:
+                if not os.path.exists(path):
+                    raise
 
 
 def makeContainingPath(filename):
@@ -236,9 +243,14 @@ def _getRealPathWindows(path):
             if str is not bytes:
                 result = result.decode("utf8")
 
-            _real_path_windows_cache[path] = os.path.join(
-                os.path.dirname(path), result.rstrip("\r\n")
-            )
+            if result.startswith("UNC\\"):
+                # Avoid network mounts being converted to UNC shared paths by newer
+                # Python versions, many tools won't work with those.
+                _real_path_windows_cache[path] = path
+            else:
+                _real_path_windows_cache[path] = os.path.join(
+                    os.path.dirname(path), result.rstrip("\r\n")
+                )
         else:
             _real_path_windows_cache[path] = path
 
@@ -270,6 +282,36 @@ def getDirectoryRealPath(path):
     return path
 
 
+def _restoreWindowsPath(orig_path, path):
+    if path.startswith("\\\\"):
+        drive, _remaining_path = os.path.splitdrive(orig_path)
+
+        if drive and not drive.startswith("\\\\"):
+            drive_real_path = os.path.realpath(drive + "\\")
+            assert path.startswith(drive_real_path)
+
+            path = drive + path[len(drive_real_path) :]
+    else:
+        path = path.strip(os.path.sep)
+
+        if os.path.sep in path:
+            dirname = os.path.dirname(path)
+            filename = os.path.basename(path)
+
+            if dirname:
+                dirname = getDirectoryRealPath(dirname)
+
+                # Drive letters do not get slashes from "os.path.join", so
+                # we inject this here and normalize the path afterwards to
+                # remove any duplication added.
+                if os.path.sep not in dirname:
+                    dirname = dirname + os.path.sep
+
+                path = os.path.normpath(os.path.join(dirname, filename))
+
+    return path
+
+
 def getFilenameRealPath(path):
     """Get os.path.realpath with Python2 and Windows symlink workaround applied.
 
@@ -288,35 +330,10 @@ def getFilenameRealPath(path):
     orig_path = path
     path = os.path.realpath(path)
 
-    # Avoid network mounts being converted to UNC shared paths ny newer
+    # Avoid network mounts being converted to UNC shared paths by newer
     # Python versions, many tools won't work with those.
-    if os.name == "nt" and path.startswith("\\\\"):
-        drive, _remaining_path = os.path.splitdrive(orig_path)
-
-        if drive:
-            drive_real_path = os.path.realpath(drive + "\\")
-            assert path.startswith(drive_real_path)
-
-            path = drive + path[len(drive_real_path) :]
-
-    # Attempt to resolve Windows symlinks older Python
     if os.name == "nt":
-        path = path.strip(os.path.sep)
-
-        if os.path.sep in path:
-            dirname = os.path.dirname(path)
-            filename = os.path.basename(path)
-
-            if dirname:
-                dirname = getDirectoryRealPath(dirname)
-
-                # Drive letters do not get slashes from "os.path.join", so
-                # we inject this here and normalize the path afterwards to
-                # remove any duplication added.
-                if os.path.sep not in dirname:
-                    dirname = dirname + os.path.sep
-
-                path = os.path.normpath(os.path.join(dirname, filename))
+        path = _restoreWindowsPath(orig_path=orig_path, path=path)
 
     return path
 
@@ -478,9 +495,9 @@ def getDllBasename(path):
         if compare_path.endswith(suffix):
             return path[: -len(suffix)]
 
-    # Linux us not case sensitive, but lets still do it properly,
-    # sometimes, it is done on non-Linux too. So we split on the
-    # normcase, but only to find out what is going on there.
+    # Linux is not case sensitive, but lets still do it properly, sometimes, it
+    # is done macOS too. So we split on the normcase, but only to find out what
+    # is going on there.
     if ".so." in compare_path:
         return path[: len(compare_path.split(".so.")[0])]
 
@@ -542,7 +559,7 @@ def listExeFilesFromDirectory(path, prefix=None, suffixes=None):
 
     # On Windows, we check exe suffixes, on other platforms we shell all filenames,
     # matching the prefix, but they have to the executable bit set.
-    if suffixes is None and isWin32OrPosixWindows():
+    if not suffixes and isWin32OrPosixWindows():
         suffixes = "exe", "bin"
 
     if suffixes:
@@ -694,6 +711,13 @@ def hasFilenameExtension(path, extensions):
         return extension in extensions
 
 
+def addFilenameExtension(path, extension):
+    if not hasFilenameExtension(path, extension):
+        path += extension
+
+    return path
+
+
 def removeDirectory(path, ignore_errors):
     """Remove a directory recursively.
 
@@ -818,6 +842,22 @@ def putTextFileContents(filename, contents, encoding=None):
             _writeContents(output_file)
 
 
+def putBinaryFileContents(filename, contents):
+    """Write a binary file from given contents.
+
+    Args:
+        filename: str with the file to be created
+        contents: bytes that should be written into the file
+
+    Returns:
+        None
+    """
+
+    with withFileLock("writing file %s" % filename):
+        with openTextFile(filename, "wb") as output_file:
+            output_file.write(contents)
+
+
 def changeTextFileContents(filename, contents, encoding=None, compare_only=False):
     """Write a text file from given contents.
 
@@ -917,10 +957,13 @@ def copyTree(source_path, dest_path):
     if python_version >= 0x380:
         # Python 3.8+ has dirs_exist_ok
         return shutil.copytree(source_path, dest_path, dirs_exist_ok=True)
+    else:
 
-    from distutils.dir_util import copy_tree
+        from distutils.dir_util import (  # Older Python only, pylint: disable=I0021,import-error
+            copy_tree,
+        )
 
-    return copy_tree(source_path, dest_path)
+        return copy_tree(source_path, dest_path)
 
 
 def copyFileWithPermissions(source_path, dest_path, dist_dir):
