@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Helper to import a file as a module.
 
 Used for Nuitka plugins and for test code.
@@ -26,6 +12,7 @@ import sys
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general
 
+from .ModuleNames import ModuleName
 from .Utils import withNoDeprecationWarning
 
 
@@ -53,7 +40,7 @@ def _importFilePy3OldWay(filename):
 
 def importFilePy2(filename):
     """Import a file for Python version 2."""
-    import imp
+    import imp  # Python2 only, pylint: disable=I0021,import-error
 
     basename = os.path.splitext(os.path.basename(filename))[0]
     return imp.load_source(basename, filename)
@@ -89,7 +76,7 @@ def getSharedLibrarySuffixes():
 
     if _shared_library_suffixes is None:
         if python_version < 0x300:
-            import imp
+            import imp  # Python2 only, pylint: disable=I0021,import-error
 
             _shared_library_suffixes = []
 
@@ -163,7 +150,10 @@ def _importFromFolder(logger, module_name, path, must_exist, message):
         del sys.path[0]
 
 
-def importFromInlineCopy(module_name, must_exist):
+_deleted_modules = {}
+
+
+def importFromInlineCopy(module_name, must_exist, delete_module=False):
     """Import a module from the inline copy stage."""
 
     folder_name = os.path.normpath(
@@ -181,7 +171,7 @@ def importFromInlineCopy(module_name, must_exist):
     elif python_version < 0x360 and os.path.exists(candidate_35):
         folder_name = candidate_35
 
-    return _importFromFolder(
+    module = _importFromFolder(
         module_name=module_name,
         path=folder_name,
         must_exist=must_exist,
@@ -189,6 +179,19 @@ def importFromInlineCopy(module_name, must_exist):
         logger=general,
     )
 
+    if delete_module and module_name in sys.modules:
+        delete_module_names = set([module_name])
+
+        for m in sys.modules:
+            if m.startswith(module_name + "."):
+                delete_module_names.add(m)
+
+        for delete_module_name in delete_module_names:
+            _deleted_modules[delete_module_name] = sys.modules[delete_module_name]
+            del sys.modules[delete_module_name]
+
+    return module
+
 
 _compile_time_modules = {}
 
@@ -220,8 +223,101 @@ def importFromCompileTime(module_name, must_exist):
 
 def isBuiltinModuleName(module_name):
     if python_version < 0x300:
-        import imp as _imp
+        import imp as _imp  # Python2 only, pylint: disable=I0021,import-error
     else:
         import _imp
 
-    return _imp.is_builtin(module_name)
+    result = _imp.is_builtin(module_name) or _imp.is_frozen(module_name)
+
+    # Some frozen modules are not actually in that list, e.g.
+    # "importlib._bootstrap_external" on Python3.10 doesn't report to
+    # "_imp.is_frozen()" above, so we check if it's already loaded and from the
+    # "FrozenImporter" by name.
+    if result is False and module_name in sys.modules:
+        module = sys.modules[module_name]
+
+        if hasattr(module, "__loader__"):
+            loader = module.__loader__
+
+            try:
+                result = loader.__name__ == "FrozenImporter"
+            except AttributeError:
+                pass
+
+    return result
+
+
+# Have a set for quicker lookups, and we cannot have "__main__" in there.
+builtin_module_names = set(
+    module_name for module_name in sys.builtin_module_names if module_name != "__main__"
+)
+
+
+def getModuleFilenameSuffixes():
+    if python_version < 0x3C0:
+        import imp  # Older Python only, pylint: disable=I0021,import-error
+
+        for suffix, _mode, module_type in imp.get_suffixes():
+            if module_type == imp.C_EXTENSION:
+                module_type = "C_EXTENSION"
+            elif module_type == imp.PY_SOURCE:
+                module_type = "PY_SOURCE"
+            elif module_type == imp.PY_COMPILED:
+                module_type = "PY_COMPILED"
+            else:
+                assert False, module_type
+
+            yield suffix, module_type
+    else:
+        import importlib.machinery
+
+        for suffix in importlib.machinery.EXTENSION_SUFFIXES:
+            yield suffix, "C_EXTENSION"
+        for suffix in importlib.machinery.SOURCE_SUFFIXES:
+            yield suffix, "PY_SOURCE"
+        for suffix in importlib.machinery.BYTECODE_SUFFIXES:
+            yield suffix, "PY_COMPILED"
+
+
+def getModuleNameAndKindFromFilenameSuffix(module_filename):
+    """Given a filename, decide the module name and kind.
+
+    Args:
+        module_name - file path of the module
+    Returns:
+        Tuple with the name of the module basename, and the kind of the
+        module derived from the file suffix. Can be None, None if is is not a
+        known file suffix.
+    Notes:
+        This doesn't handle packages at all.
+    """
+    if module_filename.endswith(".py"):
+        return ModuleName(os.path.basename(module_filename)[:-3]), "py"
+
+    if module_filename.endswith(".pyc"):
+        return ModuleName(os.path.basename(module_filename)[:-4]), "pyc"
+
+    for suffix in getSharedLibrarySuffixes():
+        if module_filename.endswith(suffix):
+            return (
+                ModuleName(os.path.basename(module_filename)[: -len(suffix)]),
+                "extension",
+            )
+
+    return None, None
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
